/* AccessibilityPoint.java

	Purpose:
		
	Description:
		
	History:
		2:30 PM 2022/9/19, Created by jumperchen

Copyright (C) 2022 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.json.JavaScriptValue;

/**
 * Options for descriptions of individual data points.
 * @author jumperchen
 * @since 10.2.1.0
 */
public class AccessibilityPoint extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		dateFormat,
		dateFormatter,
		describeNull,
		descriptionFormatter,
		descriptionFormat,
		valueDecimals,
		valueDescriptionFormat,
		valuePrefix,
		valueSuffix
	}

	/**
	 * Returns the date format to use for points on datetime axes when describing
	 * them to screen reader users.
	 *
	 * <p>Defaults to the same format as in tooltip.
	 */
	public String getDateFormat() {
		return getAttr(Attrs.dateFormat, null).asString();
	}

	/**
	 * Sets the date format to use for points on datetime axes when describing
	 * them to screen reader users.
	 *
	 * <p>Defaults to the same format as in tooltip.
	 */
	public void setDateFormat(String dateFormat) {
		setAttr(Attrs.dateFormat, dateFormat);
	}

	/**
	 * Returns the formatter function to determine the date/time format used with
	 * points on datetime axes when describing them to screen reader users.
	 * <p>Default: {@code null}
	 */
	public JavaScriptValue getDateFormatter() {
		return (JavaScriptValue) getAttr(Attrs.dateFormatter, null).asValue();
	}

	/**
	 * Sets the formatter function to determine the date/time format used with
	 * points on datetime axes when describing them to screen reader users.
	 * <p> Receives one argument, {@code point}, referring to the point to describe.
	 * Should return a date format string compatible with {@link #setDateFormat(String)}.
	 */
	public void setDateFormatter(JavaScriptValue dateFormatter) {
		setAttr(Attrs.dateFormatter, dateFormatter);
	}

	/**
	 * Returns whether or not to describe points with the value null to assistive
	 * technology, such as screen readers.
	 * <p>Default: {@code true}
	 */
	public boolean isDescribeNull() {
		return getAttr(Attrs.describeNull, true).asBoolean();
	}

	/**
	 * Sets whether or not to describe points with the value null to assistive
	 * technology, such as screen readers.
	 */
	public void setDescribeNull(boolean describeNull) {
		setAttr(Attrs.describeNull, describeNull);
	}

	/**
	 * Returns the formatter function to use instead of the default for point descriptions.
	 */
	public JavaScriptValue getDescriptionFormatter() {
		return (JavaScriptValue) getAttr(Attrs.descriptionFormatter,
				null).asValue();
	}

	/**
	 * Sets the formatter function to use instead of the default for point descriptions.
	 *
	 * <p>
	 * Receives one argument, {@code point}, referring to the point to describe.
	 * Should return a string with the description of the point for a screen reader user.
	 * If false is returned, the default formatter will be used for that point.
	 *
	 * <p>
	 * Note: Prefer using {@code accessibility.point.valueDescriptionFormat} instead if possible,
	 * as default functionality such as describing annotations will be preserved.
	 *
	 * <p>Default: {@code null}
	 */
	public void setDescriptionFormatter(JavaScriptValue descriptionFormatter) {
		setAttr(Attrs.descriptionFormatter, descriptionFormatter);
	}

	/**
	 * Returns the format to use for describing the data series group to assistive
	 * <p>Default: {@code null}
	 * @since 11.2.0.0
	 */
	public String getDescriptionFormat() {
		return getAttr(Attrs.descriptionFormat, null).asString();
	}

	/**
	 * Sets the format to use for describing the data series group to assistive
	 * @since 11.2.0.0
	 */
	public void setDescriptionFormat(String descriptionFormat) {
		setAttr(Attrs.descriptionFormat, descriptionFormat);
	}

	/**
	 * Returns decimals to use for the values in the point descriptions.
	 * <p>Default: {@code null}
	 */
	public Number getValueDecimals() {
		return getAttr(Attrs.valueDecimals, null).asNumber();
	}

	/**
	 * Sets decimals to use for the values in the point descriptions.
	 * Uses {@code tooltip.valueDecimals} if not defined.
	 */
	public void setValueDecimals(Number valueDecimals) {
		setAttr(Attrs.valueDecimals, valueDecimals);
	}

	/**
	 * Returns the format to use for describing the values of data points to assistive
	 * technology - including screen readers. The point context is available as {@code {point}}.
	 * <p>
	 * Other available context variables include {@code {index}}, {@code {value}},
	 * and {@code {xDescription}}.
	 *
	 * <p>Additionally, the series name, annotation info, and description added in
	 * {@code point.accessibility.description} is added by default if relevant.
	 * To override this, use the {@code accessibility.point.descriptionFormatter} option.
	 * <p> Default: {@code "{xDescription}{separator}{value}".}
	 */
	public String getValueDescriptionFormat() {
		return getAttr(Attrs.valueDescriptionFormat,
				"{xDescription}{separator}{value}.").asString();
	}

	/**
	 * Sets the format to use for describing the values of data points to assistive
	 * technology - including screen readers. The point context is available as {@code {point}}.
	 * <p>
	 * Other available context variables include {@code {index}}, {@code {value}},
	 * and {@code {xDescription}}.
	 *
	 * <p>Additionally, the series name, annotation info, and description added in
	 * {@code point.accessibility.description} is added by default if relevant.
	 * To override this, use the {@code accessibility.point.descriptionFormatter} option.
	 */
	public void setValueDescriptionFormat(String valueDescriptionFormat) {
		setAttr(Attrs.valueDescriptionFormat, valueDescriptionFormat);
	}

	/**
	 * Returns the prefix to add to the values in the point descriptions.
	 * Uses {@code tooltip.valuePrefix} if not defined.
	 * <p>Default: {@code null}
	 */
	public String getValuePrefix() {
		return getAttr(Attrs.valuePrefix, null).asString();
	}

	/**
	 * Sets the prefix to add to the values in the point descriptions.
	 * Uses {@code tooltip.valuePrefix} if not defined.
	 * <p>Default: {@code null}
	 */
	public void setValuePrefix(String valuePrefix) {
		setAttr(Attrs.valuePrefix, valuePrefix);
	}

	/**
	 * Returns the suffix to add to the values in the point descriptions.
	 * Uses {@code tooltip.valueSuffix} if not defined.
	 * <p>Default: {@code null}
	 */
	public String getValueSuffix() {
		return getAttr(Attrs.valueSuffix, null).asString();
	}

	/**
	 * Sets the suffix to add to the values in the point descriptions.
	 * Uses {@code tooltip.valueSuffix} if not defined.
	 * <p>Default: {@code null}
	 */
	public void setValueSuffix(String valueSuffix) {
		setAttr(Attrs.valueSuffix, valueSuffix);
	}
}
