/* AnnounceNewData.java

	Purpose:
		
	Description:
		
	History:
		11:27 AM 2022/9/19, Created by jumperchen

Copyright (C) 2022 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.json.JavaScriptValue;

/**
 * An options for announcing new data to screen reader users.
 * Useful for dynamic data applications and drilldown.
 * <p>Keep in mind that frequent announcements will not be useful to users, as they
 * won't have time to explore the new data. For these applications, consider making
 * snapshots of the data accessible, and do the announcements in batches.
 * @author jumperchen
 * @since 10.2.1.0
 */
public class AnnounceNewData extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		announcementFormatter,
		enabled,
		interruptUser,
		minAnnounceInterval
	}

	/**
	 * Returns the optional formatter callback for the announcement
	 * <p>Default: {@code null}
	 */
	public JavaScriptValue getAnnouncementFormatter() {
		return (JavaScriptValue) getAttr(Attrs.announcementFormatter, null).asValue();
	}

	/**
	 * Sets the optional formatter callback for the announcement. Receives up to
	 * three arguments. The first argument is always an array of all series that
	 * received updates. If an announcement is already queued, the series that received
	 * updates for that announcement are also included in this array. The second argument
	 * is provided if chart.addSeries was called, and there is a new series.
	 * In that case, this argument is a reference to the new series. The third argument,
	 * similarly, is provided if series.addPoint was called, and there is a new point.
	 * In that case, this argument is a reference to the new point.
	 *
	 * <p>
	 * The function should return a string with the text to announce to the user. Return empty string to not announce anything.
	 * Return false to use the default announcement format.
	 *
	 * @param announcementFormatter
	 */
	public void setAnnouncementFormatter(JavaScriptValue announcementFormatter) {
		setAttr(Attrs.announcementFormatter, announcementFormatter);
	}

	/**
	 * Returns whether to enable announcing new data to screen reader users.
	 * <p>Default: {@code false}
	 */
	public boolean isEnabled() {
		return getAttr(Attrs.enabled, false).asBoolean();
	}

	/**
	 * Sets whether to enable announcing new data to screen reader users.
	 * <p>Default: {@code false}
	 */
	public void setEnabled(boolean enabled) {
		setAttr(Attrs.enabled, enabled);
	}

	/**
	 * Returns whether or not the announcements should interrupt the screen reader.
	 * If not enabled, the user will be notified once idle. It is recommended not
	 * to enable this setting unless there is a specific reason to do so.
	 * <p>Default: {@code false}
	 */
	public boolean isInterruptUser() {
		return getAttr(Attrs.interruptUser, false).asBoolean();
	}

	/**
	 * Sets whether or not the announcements should interrupt the screen reader.
	 * If not enabled, the user will be notified once idle. It is recommended not
	 * to enable this setting unless there is a specific reason to do so.
	 * <p>Default: {@code false}
	 */
	public void setInterruptUser(boolean interruptUser) {
		setAttr(Attrs.interruptUser, interruptUser);
	}

	/**
	 * Returns the minimum interval between announcements in milliseconds.
	 * If new data arrives before this amount of time has passed, it is queued for
	 * announcement. If another new data event happens while an announcement is queued,
	 * the queued announcement is dropped, and the latest announcement is queued instead.
	 * Set to 0 to allow all announcements, but be warned that frequent announcements
	 * are disturbing to users.
	 * <p>Default: 5000</p>
	 */
	public Number getMinAnnounceInterval() {
		return getAttr(Attrs.minAnnounceInterval, 5000).asNumber();
	}

	/**
	 * Sets the minimum interval between announcements in milliseconds.
	 * If new data arrives before this amount of time has passed, it is queued for
	 * announcement. If another new data event happens while an announcement is queued,
	 * the queued announcement is dropped, and the latest announcement is queued instead.
	 * Set to 0 to allow all announcements, but be warned that frequent announcements
	 * are disturbing to users.
	 * <p>Default: 5000</p>
	 */
	public void setMinAnnounceInterval(Number minAnnounceInterval) {
		setAttr(Attrs.minAnnounceInterval, minAnnounceInterval);
	}
}
