/* BubbleLegend.java

	Purpose:
		
	Description:
		
	History:
		Thu Nov 14 10:50:01 CST 2019, Created by rudyhuang

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.chart.util.MapsHelper;
import org.zkoss.json.JavaScriptValue;
import org.zkoss.lang.Generics;

/**
 * The bubble legend is an additional element in legend which presents the scale of the bubble series.
 * Individual bubble ranges can be defined by user or calculated from series.
 * In the case of automatically calculated ranges, a 1px margin of error is permitted.
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 *
 * @author rudyhuang
 * @since 7.2.1.0
 */
public class BubbleLegend extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		borderColor, borderWidth, className, color, connectorClassName, connectorColor,
		connectorDistance, connectorWidth, enabled, labels, legendIndex,
		maxSize, minSize, ranges, sizeBy, sizeByAbsoluteValue, zIndex, zThreshold
	}

	/**
	 * Returns the color of the ranges borders, can be also defined for an individual range.
	 * <p>
	 * Default: null
	 */
	public Color getBorderColor() {
		return (Color) getAttr(Attrs.borderColor, null).asValue();
	}

	/**
	 * Sets the color of the ranges borders, can be also defined for an individual range.
	 */
	public void setBorderColor(Color color) {
		setAttr(Attrs.borderColor, color);
	}

	/**
	 * Sets the color of the ranges borders, can be also defined for an individual range.
	 */
	public void setBorderColor(String color) {
		setBorderColor(new Color(color));
	}

	/**
	 * Sets the color of the ranges borders, can be also defined for an individual range.
	 */
	public void setBorderColor(LinearGradient color) {
		setBorderColor(new Color(color));
	}

	/**
	 * Sets the color of the ranges borders, can be also defined for an individual range.
	 */
	public void setBorderColor(RadialGradient color) {
		setBorderColor(new Color(color));
	}

	/**
	 * Returns the width of the ranges borders in pixels, can be also defined for an individual range.
	 * <p>
	 * Default: 2
	 */
	public Number getBorderWidth() {
		return getAttr(Attrs.borderWidth, 2).asNumber();
	}

	/**
	 * Sets the width of the ranges borders in pixels, can be also defined for an individual range.
	 */
	public void setBorderWidth(Number borderWidth) {
		setAttr(Attrs.borderWidth, borderWidth, 2);
	}

	/**
	 * Returns an additional class name to apply to the bubble legend' circle graphical elements.
	 * This option does not replace default class names of the graphical element.
	 * <p>
	 * Default: ""
	 */
	public String getClassName() {
		return getAttr(Attrs.className, "").asString();
	}

	/**
	 * Sets an additional class name to apply to the bubble legend' circle graphical elements.
	 * This option does not replace default class names of the graphical element.
	 */
	public void setClassName(String className) {
		setAttr(Attrs.className, className, "");
	}

	/**
	 * Returns the main color of the bubble legend.
	 * Applies to ranges, if individual color is not defined.
	 * <p>
	 * Default: null
	 */
	public Color getColor() {
		return (Color) getAttr(Attrs.color, null).asValue();
	}

	/**
	 * Sets the main color of the bubble legend.
	 * Applies to ranges, if individual color is not defined.
	 */
	public void setColor(Color color) {
		setAttr(Attrs.color, color);
	}

	/**
	 * Sets the main color of the bubble legend.
	 * Applies to ranges, if individual color is not defined.
	 */
	public void setColor(String color) {
		setColor(new Color(color));
	}

	/**
	 * Sets the main color of the bubble legend.
	 * Applies to ranges, if individual color is not defined.
	 */
	public void setColor(LinearGradient color) {
		setColor(new Color(color));
	}

	/**
	 * Sets the main color of the bubble legend.
	 * Applies to ranges, if individual color is not defined.
	 */
	public void setColor(RadialGradient color) {
		setColor(new Color(color));
	}

	/**
	 * Returns an additional class name to apply to the bubble legend's connector graphical elements.
	 * This option does not replace default class names of the graphical element.
	 * <p>
	 * Default: ""
	 */
	public String getConnectorClassName() {
		return getAttr(Attrs.connectorClassName, "").asString();
	}

	/**
	 * Sets an additional class name to apply to the bubble legend's connector graphical elements.
	 * This option does not replace default class names of the graphical element.
	 */
	public void setConnectorClassName(String connectorClassName) {
		setAttr(Attrs.connectorClassName, connectorClassName, "");
	}

	/**
	 * Returns the color of the connector, can be also defined for an individual range.
	 * <p>
	 * Default: null
	 */
	public Color getConnectorColor() {
		return (Color) getAttr(Attrs.connectorColor, null).asValue();
	}

	/**
	 * Sets the color of the connector, can be also defined for an individual range.
	 */
	public void setConnectorColor(Color connectorColor) {
		setAttr(Attrs.connectorColor, connectorColor);
	}

	/**
	 * Sets the color of the connector, can be also defined for an individual range.
	 */
	public void setConnectorColor(String connectorColor) {
		setConnectorColor(new Color(connectorColor));
	}

	/**
	 * Sets the color of the connector, can be also defined for an individual range.
	 */
	public void setConnectorColor(LinearGradient connectorColor) {
		setConnectorColor(new Color(connectorColor));
	}

	/**
	 * Sets the color of the connector, can be also defined for an individual range.
	 */
	public void setConnectorColor(RadialGradient connectorColor) {
		setConnectorColor(new Color(connectorColor));
	}

	/**
	 * Returns the length of the connectors in pixels.
	 * If labels are centered, the distance is reduced to 0.
	 * <p>
	 * Default: 60
	 */
	public Number getConnectorDistance() {
		return getAttr(Attrs.connectorDistance, 60).asNumber();
	}

	/**
	 * Sets the length of the connectors in pixels.
	 * If labels are centered, the distance is reduced to 0.
	 */
	public void setConnectorDistance(Number connectorDistance) {
		setAttr(Attrs.connectorDistance, connectorDistance, 60);
	}

	/**
	 * Returns the width of the connectors in pixels.
	 * <p>
	 * Default: 1
	 */
	public Number getConnectorWidth() {
		return getAttr(Attrs.connectorWidth, 1).asNumber();
	}

	/**
	 * Sets the width of the connectors in pixels.
	 */
	public void setConnectorWidth(Number connectorWidth) {
		setAttr(Attrs.connectorWidth, connectorWidth, 1);
	}


	/**
	 * Returns enable or disable the bubble legend.
	 * <p>
	 * Default: false
	 */
	public boolean isEnabled() {
		return getAttr(Attrs.enabled, false).asBoolean();
	}

	/**
	 * Sets enable or disable the bubble legend.
	 */
	public void setEnabled(boolean enabled) {
		setAttr(Attrs.enabled, enabled, false);
	}

	/**
	 * Returns options for the bubble legend labels if any. Otherwise, create a new one.
	 */
	public Labels getLabels() {
		Labels labels = (Labels) getAttr(Attrs.labels);
		if (labels == null) {
			labels = new Labels();
			setLabels(labels);
		}
		return labels;
	}

	/**
	 * Sets options for the bubble legend labels.
	 */
	public void setLabels(Labels labels) {
		setAttr(Attrs.labels, labels);
	}

	/**
	 * Returns the position of the bubble legend in the legend.
	 * <p>
	 * Default: 0
	 */
	public Number getLegendIndex() {
		return getAttr(Attrs.legendIndex, 0).asNumber();
	}

	/**
	 * Sets the position of the bubble legend in the legend.
	 */
	public void setLegendIndex(Number legendIndex) {
		setAttr(Attrs.legendIndex, legendIndex, 0);
	}

	/**
	 * Returns maximum bubble legend range size.
	 * If values for ranges are not specified, the minSize and the maxSize are calculated from bubble series.
	 * <p>
	 * Default: 60
	 */
	public Number getMaxSize() {
		return getAttr(Attrs.maxSize, 60).asNumber();
	}

	/**
	 * Sets maximum bubble legend range size.
	 * If values for ranges are not specified, the minSize and the maxSize are calculated from bubble series.
	 */
	public void setMaxSize(Number maxSize) {
		setAttr(Attrs.maxSize, maxSize, 60);
	}

	/**
	 * Returns minimum bubble legend range size.
	 * If values for ranges are not specified, the minSize and the maxSize are calculated from bubble series.
	 * <p>
	 * Default: 10
	 */
	public Number getMinSize() {
		return getAttr(Attrs.minSize, 10).asNumber();
	}

	/**
	 * Sets minimum bubble legend range size.
	 * If values for ranges are not specified, the minSize and the maxSize are calculated from bubble series.
	 */
	public void setMinSize(Number minSize) {
		setAttr(Attrs.minSize, minSize, 10);
	}

	/**
	 * Returns the options for specific range.
	 * One range consists of bubble, label and connector.
	 */
	public List<Range> getRanges() {
		return Generics.cast(getAttr(Attrs.ranges));
	}

	/**
	 * Sets the options for specific range.
	 * One range consists of bubble, label and connector.
	 */
	public void setRanges(List<Range> ranges) {
		setAttr(Attrs.ranges, ranges);
	}

	/**
	 * Sets the options for specific range.
	 * One range consists of bubble, label and connector.
	 */
	public void setRanges(Range... ranges) {
		setRanges(new ArrayList<Range>(Arrays.asList(ranges)));
	}

	/**
	 * Returns whether the bubble legend range value should be represented
	 * by the area or the width of the bubble.
	 * The default, area, corresponds best to the human perception of the size of each bubble.
	 * <p>
	 * Default: "area"
	 */
	public String getSizeBy() {
		return getAttr(Attrs.sizeBy, "area").asString();
	}

	/**
	 * Sets whether the bubble legend range value should be represented
	 * by the area or the width of the bubble.
	 * The default, area, corresponds best to the human perception of the size of each bubble.
	 */
	public void setSizeBy(String sizeBy) {
		setAttr(Attrs.sizeBy, sizeBy);
	}

	/**
	 * When this is true, the absolute value of z determines the size of the bubble.
	 * This means that with the default zThreshold of 0,
	 * a bubble of value -1 will have the same size as a bubble of value 1,
	 * while a bubble of value 0 will have a smaller size according to minSize.
	 * <p>
	 * Default: false.
	 */
	public boolean isSizeByAbsoluteValue() {
		return getAttr(Attrs.sizeByAbsoluteValue, false).asBoolean();
	}

	/**
	 * When this is true, the absolute value of z determines the size of the bubble.
	 * This means that with the default zThreshold of 0,
	 * a bubble of value -1 will have the same size as a bubble of value 1,
	 * while a bubble of value 0 will have a smaller size according to minSize.
	 */
	public void setSizeByAbsoluteValue(boolean sizeByAbsoluteValue) {
		setAttr(Attrs.sizeByAbsoluteValue, sizeByAbsoluteValue, false);
	}

	/**
	 * Returns the visual z index of the bubble legend.
	 * <p>
	 * Default: 1
	 */
	public Number getZIndex() {
		return getAttr(Attrs.zIndex, 1).asNumber();
	}

	/**
	 * Sets the visual z index of the bubble legend.
	 */
	public void setZIndex(Number zIndex) {
		setAttr(Attrs.zIndex, zIndex, 1);
	}

	/**
	 * Returns the z threshold.
	 * Ranges with with lower value than zThreshold, are skipped.
	 * <p>
	 * Default: 0
	 */
	public Number getZThreshold() {
		return getAttr(Attrs.zThreshold, 0).asNumber();
	}

	/**
	 * Sets the z threshold.
	 * Ranges with with lower value than zThreshold, are skipped.
	 */
	public void setZThreshold(Number zThreshold) {
		setAttr(Attrs.zThreshold, zThreshold, 0);
	}

	/**
	 * Options for specific range. One range consists of bubble, label and connector.
	 *
	 * @author rudyhuang
	 * @since 7.2.1.0
	 */
	public static class Range extends Optionable {
		private enum Attrs implements PlotAttribute, DynamicalAttribute {
			borderColor, color, connectorColor, value
		}

		/**
		 * Default constructor.
		 */
		public Range() {
		}

		/**
		 * Constructs with value.
		 */
		public Range(Number value) {
			setValue(value);
		}

		/**
		 * Returns the color of the border for individual range.
		 * <p>
		 * Default: null
		 */
		public Color getBorderColor() {
			return (Color) getAttr(Attrs.borderColor, null).asValue();
		}

		/**
		 * Sets the color of the border for individual range.
		 */
		public void setBorderColor(Color color) {
			setAttr(Attrs.borderColor, color);
		}

		/**
		 * Sets the color of the border for individual range.
		 */
		public void setBorderColor(String color) {
			setBorderColor(new Color(color));
		}

		/**
		 * Sets the color of the border for individual range.
		 */
		public void setBorderColor(LinearGradient color) {
			setBorderColor(new Color(color));
		}

		/**
		 * Sets the color of the border for individual range.
		 */
		public void setBorderColor(RadialGradient color) {
			setBorderColor(new Color(color));
		}

		/**
		 * Returns the color of the bubble for individual range.
		 * <p>
		 * Default: null
		 */
		public Color getColor() {
			return (Color) getAttr(Attrs.color, null).asValue();
		}

		/**
		 * Sets the color of the bubble for individual range.
		 */
		public void setColor(Color color) {
			setAttr(Attrs.color, color);
		}

		/**
		 * Sets the color of the bubble for individual range.
		 */
		public void setColor(String color) {
			setColor(new Color(color));
		}

		/**
		 * Sets the color of the bubble for individual range.
		 */
		public void setColor(LinearGradient color) {
			setColor(new Color(color));
		}

		/**
		 * Sets the color of the bubble for individual range.
		 */
		public void setColor(RadialGradient color) {
			setColor(new Color(color));
		}

		/**
		 * Returns the color of the connector for individual range.
		 * <p>
		 * Default: null
		 */
		public Color getConnectorColor() {
			return (Color) getAttr(Attrs.connectorColor, null).asValue();
		}

		/**
		 * Sets the color of the connector for individual range.
		 */
		public void setConnectorColor(Color connectorColor) {
			setAttr(Attrs.connectorColor, connectorColor);
		}

		/**
		 * Sets the color of the connector for individual range.
		 */
		public void setConnectorColor(String connectorColor) {
			setConnectorColor(new Color(connectorColor));
		}

		/**
		 * Sets the color of the connector for individual range.
		 */
		public void setConnectorColor(LinearGradient connectorColor) {
			setConnectorColor(new Color(connectorColor));
		}

		/**
		 * Sets the color of the connector for individual range.
		 */
		public void setConnectorColor(RadialGradient connectorColor) {
			setConnectorColor(new Color(connectorColor));
		}

		/**
		 * Returns the range size value, similar to bubble Z data.
		 * <p>
		 * Default: null
		 */
		public Number getValue() {
			return getAttr(Attrs.value, null).asNumber();
		}

		/**
		 * Sets the range size value, similar to bubble Z data.
		 */
		public void setValue(Number value) {
			setAttr(Attrs.value, value);
		}
	}

	/**
	 * Options for the bubble legend labels.
	 *
	 * @author rudyhuang
	 * @since 7.2.1.0
	 */
	public static class Labels extends Optionable {
		private enum Attrs implements PlotAttribute, DynamicalAttribute {
			align, allowOverlap, className, format, formatter, style, x, y
		}

		/**
		 * Returns the alignment of the labels compared to the bubble legend.
		 * Can be one of left, center or right.
		 * <p>
		 * Default: "right"
		 */
		public String getAlign() {
			return getAttr(Attrs.align, "right").asString();
		}

		/**
		 * Sets the alignment of the labels compared to the bubble legend.
		 * Can be one of left, center or right.
		 */
		public void setAlign(String align) {
			if (!("left".equals(align) || "center".equals(align) || "right".equals(align)))
				throw new IllegalArgumentException("Unsupported align: [" + align + "]");
			setAttr(Attrs.align, align, "right");
		}

		/**
		 * Returns whether to allow data labels to overlap.
		 * <p>
		 * Default: false.
		 */
		public boolean isAllowOverlap() {
			return getAttr(Attrs.allowOverlap, false).asBoolean();
		}
		/**
		 * Sets whether to allow data labels to overlap.
		 */
		public void setAllowOverlap(Boolean allowOverlap) {
			setAttr(Attrs.allowOverlap, allowOverlap, false);
		}

		/**
		 * Returns an additional class name to apply to the bubble legend label graphical elements.
		 * This option does not replace default class names of the graphical element.
		 * <p>
		 * Default: ""
		 */
		public String getClassName() {
			return getAttr(Attrs.className, "").asString();
		}

		/**
		 * Sets an additional class name to apply to the bubble legend label graphical elements.
		 * This option does not replace default class names of the graphical element.
		 */
		public void setClassName(String className) {
			setAttr(Attrs.className, className, "");
		}

		/**
		 * Returns a <a href="https://www.highcharts.com/docs/chart-concepts/labels-and-string-formatting">format string</a> for the bubble legend labels.
		 * Available variables are the same as for formatter.
		 * <p>
		 * Default: ""
		 */
		public String getFormat() {
			return getAttr(Attrs.format, "").asString();
		}

		/**
		 * Sets a <a href="https://www.highcharts.com/docs/chart-concepts/labels-and-string-formatting">format string</a> for the bubble legend labels.
		 * Available variables are the same as for formatter.
		 */
		public void setFormat(String format) {
			setAttr(Attrs.format, format, "");
		}

		/**
		 * Returns the callback JavaScript function to format the bubble legend labels.
		 * Available <code>this</code> properties are:
		 * <ul>
		 *     <li><code>this.value</code>: The bubble value.</li>
		 *     <li><code>this.radius</code>: The radius of the bubble range.</li>
		 *     <li><code>this.center</code>: The center y position of the range.</li>
		 * </ul>
		 * <p>
		 * Default: null
		 */
		public JavaScriptValue getFormatter() {
			return (JavaScriptValue) getAttr(Attrs.formatter, new JavaScriptValue("function () { return defined(this.y) ? this.y : 'Annotation label'; }")).asValue();
		}

		/**
		 * Sets the callback JavaScript function to format the bubble legend labels.
		 * Available <code>this</code> properties are:
		 * <ul>
		 *     <li><code>this.value</code>: The bubble value.</li>
		 *     <li><code>this.radius</code>: The radius of the bubble range.</li>
		 *     <li><code>this.center</code>: The center y position of the range.</li>
		 * </ul>
		 */
		public void setFormatter(JavaScriptValue formatter) {
			setAttr(Attrs.formatter, formatter);
		}

		/**
		 * Returns CSS styles for the labels.
		 * <p>
		 * Default: empty
		 * @param <K> type for the key
		 * <p>
		 * if style was set using {@link #setStyle(String)}, type will be String
		 * <p>
		 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
		 * <p>
		 * if style was not set, default style will be applied, and type will be String
		 * @param <V> type for the value
		 * <p>
		 * if style was set using {@link #setStyle(String)}, type will be String
		 * <p>
		 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
		 * <p>
		 * if style was not set, default style will be applied, and type will be String
		 */
		public <K, V> Map<K, V> getStyle() {
			if (!containsKey(Attrs.style)) {
				setStyle("");
			}
			return Generics.cast(getAttr(Attrs.style));
		}

		/**
		 * Sets CSS styles for the labels.
		 * <p>
		 * This method allows <b>simple</b> settings like
		 * <pre>{@code
		 * label.setStyle("fontWeight: 'bold'; color: 'white'; textShadow: '0px 1px 2px black'");
		 * }</pre>
		 * <p>
		 * In the format of CSS key:value, and ";" separating multiple key:value pairs,
		 * if you have more a complex CSS styling, we recommend using {@link #setStyle(Map)}
		 * @param style one or many CSS key:value pairs in a single line string format
		 */
		public void setStyle(String style) {
			setStyle(MapsHelper.parse(new LinkedHashMap<String, String>(), style, ':', ';', '\''));
		}

		/**
		 * Sets CSS styles for the labels.
		 * <p>
		 * This method allows <b>complex</b> styling like:
		 * <pre>
		 * {@code
		 * Map<String, Object> styles = new HashMap<String, Object>();
		 * styles.put("fill", "#E8E8EA");
		 * styles.put("stroke", "#6E6E78");
		 * styles.put("stroke-width", "1");
		 * styles.put("r", "0");
		 * Map<String, Object> states = new HashMap<String, Object>();
		 * Map<String, String> hover = new HashMap<String, String>();
		 * hover.put("fill", "#F4F4F6");
		 * hover.put("stroke", "#6E6E78");
		 * hover.put("stroke-width", "1");
		 * hover.put("r", "0");
		 * states.put("hover", hover);
		 * styles.put("states", states);
		 * Map<String, String> style = new HashMap<String, String>();
		 * style.put("color", "#4D5663");
		 * style.put("fontSize", "12px");
		 * style.put("fontFamily", "'Arial'");
		 * style.put("fontWeight", "normal");
		 * style.put("padding", "10px");
		 * styles.put("style", style);
		 * }</pre>
		 * which will generate the final results like:
		 * <pre>{@code
		 * {
		 *     "fill": "#E8E8EA",
		 *     "stroke": "#6E6E78",
		 *     "stroke-width": "1",
		 *     "r": "0",
		 *     "states":
		 *     {
		 *         "hover":
		 *         {
		 *             "fill": "#F4F4F6",
		 *             "stroke": "#6E6E78",
		 *             "stroke-width": "1",
		 *             "r": "0"
		 *         }
		 *     },
		 *     "style":
		 *     {
		 *         "color": "#4D5663",
		 *         "fontSize": "12px",
		 *         "fontFamily": "'Arial'",
		 *         "fontWeight": "normal",
		 *         "padding": "10px"
		 *     }
		 * }
		 * }</pre>
		 * @param <K> Type for the key, usually String
		 * @param <V> Type for the value, usually String. For nested maps, use Object or any other type.
		 * @param style the style object
		 */
		public <K, V> void setStyle(Map<K, V> style) {
			setAttr(Attrs.style, style, NOT_NULL_VALUE);
		}

		/**
		 * Returns the x position offset of the label relative to the connector.
		 * <p> Default: 0
		 */
		public Number getX() {
			return getAttr(Attrs.x, 0).asNumber();
		}

		/**
		 * Sets the x position offset of the label relative to the connector.
		 */
		public void setX(Number x) {
			setAttr(Attrs.x, x, 0);
		}

		/**
		 * Returns the y position offset of the label relative to the connector.
		 * <p> Default: 0
		 */
		public Number getY() {
			return getAttr(Attrs.y, 0).asNumber();
		}

		/**
		 * Sets the y position offset of the label relative to the connector.
		 */
		public void setY(Number y) {
			setAttr(Attrs.y, y, 0);
		}
	}
}
