package org.zkoss.chart;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import org.zkoss.chart.util.DynamicalAttribute;


/**
 * A Color Axis used for heat map plots only
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 * 
 * @author hanhsu
 */
public class ColorAxis extends YAxis {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		accessibility,
		dataClassColor,
		dataClasses,
		layout,
		showInLegend
	}


	/**
	 * Returns an accessibility options for an axis, if null, it will create new one.
	 * @since 10.2.1.0
	 */
	public AxisAccessibility getAccessibility() {
		return (AxisAccessibility) getAttr(Attrs.accessibility, AxisAccessibility.class).asValue();
	}

	/**
	 * Sets an accessibility options for an axis.
	 * @since 10.2.1.0
	 */
	public void setAccessibility(AxisAccessibility accessibility) {
		setAttr(Attrs.accessibility, accessibility);
	}

	/**
	 * Determines how to set each data class' color if no individual color is set.
	 * The default value, tween, computes intermediate colors between minColor and maxColor.
	 * The other possible value, category, pulls colors from the global or chart specific colors array.
	 * <p>
	 * Default: "tween"
	 * @since 7.2.1.0
	 */
	public String getDataClassColor() {
		return getAttr(Attrs.dataClassColor, "tween").asString();
	}

	/**
	 * Determines how to set each data class' color if no individual color is set.
	 * The default value, tween, computes intermediate colors between minColor and maxColor.
	 * The other possible value, category, pulls colors from the global or chart specific colors array.
	 * @since 7.2.1.0
	 */
	public void setDataClassColor(String dataClassColor) {
		setAttr(Attrs.dataClassColor, dataClassColor);
	}

	/**
	 * Returns a list of data classes or ranges for the choropleth map, if any.
	 * Otherwise, create a new one.
	 * If none given, the color axis is scalar and values are distributed as
	 * a gradient between the minimum and maximum colors.
	 * @since 7.2.1.0
	 */
	public List<DataClass> getDataClasses() {
		List<DataClass> dataClasses = (List<DataClass>) getAttr(Attrs.dataClasses);
		if (dataClasses == null) {
			dataClasses = new LinkedList<DataClass>();
			setDataClasses(dataClasses);
		}
		return dataClasses;
	}

	/**
	 * Sets a list of data classes or ranges for the choropleth map.
	 * @since 7.2.1.0
	 */
	public void setDataClasses(List<DataClass> dataClasses) {
		setAttr(Attrs.dataClasses, dataClasses);
	}

	/**
	 * Sets a list of data classes or ranges for the choropleth map.
	 * @since 7.2.1.0
	 */
	public void setDataClasses(DataClass... dataClasses) {
		setAttr(Attrs.dataClasses, new LinkedList<DataClass>(Arrays.asList(dataClasses)));
	}

	/**
	 * Returns the layout of the color axis.
	 * Can be 'horizontal' or 'vertical'.
	 * If none given, the color axis has the same layout as the legend.
	 * <p>
	 * Default: null
	 * @since 7.2.1.0
	 */
	public String getLayout() {
		return getAttr(Attrs.layout, null).asString();
	}

	/**
	 * Sets the layout of the color axis.
	 * Can be 'horizontal' or 'vertical'.
	 * If none given, the color axis has the same layout as the legend.
	 * @since 7.2.1.0
	 */
	public void setLayout(String layout) {
		if (layout != null && !("horizontal".equals(layout) || "vertical".equals(layout)))
			throw new IllegalArgumentException("Unsupported layout: [ " + layout + " ]");
		setAttr(Attrs.layout, layout);
	}

	/**
	 * Returns whether to display the colorAxis in the legend.
	 * <p>
	 * Default: true
	 * @since 7.2.1.0
	 */
	public boolean isShowInLegend() {
		return getAttr(Attrs.showInLegend, true).asBoolean();
	}

	/**
	 * Sets whether to display the colorAxis in the legend.
	 * @since 7.2.1.0
	 */
	public void setShowInLegend(boolean showInLegend) {
		setAttr(Attrs.showInLegend, showInLegend, true);
	}
}
