/* DefaultSpeechOptions.java

        Purpose:
                
        Description:
                
        History:
                Fri Sep 15 14:40:31 CST 2023, Created by jamson

Copyright (C) 2023 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import org.zkoss.chart.util.DynamicalAttribute;

/**
 * Default sonification options for all speech tracks.
 * <p>If specific options are also set on individual tracks or per series, those will override these options.
 * @author Jamson Chan
 * @since 11.2.0.0
 */
public class DefaultSpeechOptions extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		activeWhen,
		language,
		mapping,
		pointGrouping,
		preferredVoice,
		showPlayMarker,
		type
	}

	/**
	 * Returns the ActiveWhen object.
	 * <p>
	 * Define a condition for when a track should be active and not.
	 * <p>
	 * Can either be a function callback or a configuration object.
	 * <p>
	 * If a function is used, it should return a <code>boolean</code> for whether or not the track should be active.
	 * The function is called for each audio event, and receives a parameter object with <code>time</code>,
	 * and potentially <code>point</code> and <code>value</code> properties depending on the track.
	 * <code>point</code> is available if the audio event is related to a data point.
	 * <code>value</code> is available if the track is used as a context track, and <code>valueInterval</code> is used.
	 */
	public ActiveWhen getActiveWhen() {
		ActiveWhen activeWhen = (ActiveWhen) getAttr(Attrs.activeWhen);
		if (activeWhen == null) {
			activeWhen = new ActiveWhen();
			setActiveWhen(activeWhen);
		}
		return activeWhen;
	}

	/**
	 * Sets the ActiveWhen object.
	 * <p>
	 * Define a condition for when a track should be active and not.
	 * <p>
	 * Can either be a function callback or a configuration object.
	 * <p>
	 * If a function is used, it should return a <code>boolean</code> for whether or not the track should be active.
	 * The function is called for each audio event, and receives a parameter object with <code>time</code>,
	 * and potentially <code>point</code> and <code>value</code> properties depending on the track.
	 * <code>point</code> is available if the audio event is related to a data point.
	 * <code>value</code> is available if the track is used as a context track, and <code>valueInterval</code> is used.
	 */
	public void setActiveWhen(ActiveWhen activeWhen) {
		setAttr(Attrs.activeWhen, activeWhen);
	}

	/**
	 * Returns the language to speak in for speech tracks, as an IETF BCP 47 language tag.
	 * <p>Default: {@code "en-US"}
	 */
	public String getLanguage() {
		return getAttr(Attrs.language, "en-US").asString();
	}

	/**
	 * Sets the language to speak in for speech tracks, as an IETF BCP 47 language tag.
	 */
	public void setLanguage(String language) {
		setAttr(Attrs.language, language, "en-US");
	}

	/**
	 * Returns the Mapping object.
	 * <p>Mapping options for the audio parameters.
	 * <p>All parameters can be either:
	 * <ul>
	 *     <li>A string, referencing a point property to map to.</li>
	 *     <li>A number, setting the value of the audio parameter directly.</li>
	 *     <li>A callback function, returning the value programmatically.</li>
	 *     <li>An object defining detailed configuration of the mapping.</li>
	 * </ul>
	 * If a function is used, it should return the desired value for the audio parameter.
	 * The function is called for each audio event to be played,
	 * and receives a context object parameter with time,
	 * and potentially point and value depending on the track.
	 * point is available if the audio event is related to a data point,
	 * and value is available if the track is used for a context track using valueInterval.
	 */
	public Mapping getMapping() {
		Mapping mapping = (Mapping) getAttr(Attrs.mapping);
		if (mapping == null) {
			mapping = new Mapping();
			setMapping(mapping);
		}
		return mapping;
	}

	/**
	 * Sets the Mapping object.
	 * <p>Mapping options for the audio parameters.
	 * <p>All parameters can be either:
	 * <ul>
	 *     <li>A string, referencing a point property to map to.</li>
	 *     <li>A number, setting the value of the audio parameter directly.</li>
	 *     <li>A callback function, returning the value programmatically.</li>
	 *     <li>An object defining detailed configuration of the mapping.</li>
	 * </ul>
	 * If a function is used, it should return the desired value for the audio parameter.
	 * The function is called for each audio event to be played,
	 * and receives a context object parameter with time,
	 * and potentially point and value depending on the track.
	 * point is available if the audio event is related to a data point,
	 * and value is available if the track is used for a context track using valueInterval.
	 */
	public void setMapping(Mapping mapping) {
		setAttr(Attrs.mapping, mapping);
	}

	/**
	 * Returns the options for point grouping, specifically for instrument tracks.
	 */
	public PointGrouping getPointGrouping() {
		PointGrouping pointGrouping = (PointGrouping) getAttr(Attrs.pointGrouping);
		if (pointGrouping == null) {
			pointGrouping = new PointGrouping();
			setPointGrouping(pointGrouping);
		}
		return pointGrouping;
	}

	/**
	 * Sets the options for point grouping, specifically for instrument tracks.
	 */
	public void setPointGrouping(PointGrouping pointGrouping) {
		setAttr(Attrs.pointGrouping, pointGrouping);
	}

	/**
	 * Returns the name of the voice synthesis to prefer for speech tracks.
	 * <p>If not available, falls back to the default voice for the selected language.
	 * <p>Different platforms provide different voices for web speech synthesis.
	 * <p>Default: {@code null}
	 */
	public String getPreferredVoice() {
		return getAttr(Attrs.preferredVoice, null).asString();
	}

	/**
	 * Sets the name of the voice synthesis to prefer for speech tracks.
	 * <p>If not available, falls back to the default voice for the selected language.
	 * <p>Different platforms provide different voices for web speech synthesis.
	 */
	public void setPreferredVoice(String preferredVoice) {
		setAttr(Attrs.preferredVoice, preferredVoice, null);
	}


	/**
	 * Returns whether to show play marker (tooltip and/or crosshair) for a track.
	 * <p>Default: {@code true}
	 */
	public boolean isShowPlayMarker() {
		return getAttr(Attrs.showPlayMarker, true).asBoolean();
	}

	/**
	 * Sets whether to show play marker (tooltip and/or crosshair) for a track.
	 */
	public void setShowPlayMarker(Boolean showPlayMarker) {
		setAttr(Attrs.showPlayMarker, showPlayMarker, true);
	}

	/**
	 * Returns the type of track. Always <code>"instrument"</code> for instrument tracks,
	 * and "speech" for <code>speech</code> tracks.
	 * <p>Default: {@code "speech"}
	 */
	public String getType() {
		return getAttr(Attrs.type, "speech").asString();
	}

	/**
	 * Sets the type of track. Always <code>"instrument"</code> for instrument tracks,
	 * and "speech" for <code>speech</code> tracks.
	 */
	public void setType(String type) {
		setAttr(Attrs.type, type, "speech");
	}

}
