/* GlobalTrack.java

        Purpose:
                
        Description:
                
        History:
                Fri Sep 15 14:41:07 CST 2023, Created by jamson

Copyright (C) 2023 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.json.JavaScriptValue;

/**
 * Global tracks to add to every series.
 * <p>Defined as an array of either instrument or speech tracks, or a combination.
 * @author Jamson Chan
 * @since 11.2.0.0
 */
public class GlobalTrack extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		activeWhen,
		instrument,
		mapping,
		midiName,
		pointGrouping,
		roundToMusicalNotes,
		showPlayMarker,
		type
	}

	/**
	 * Returns the ActiveWhen object.
	 * <p>
	 * Define a condition for when a track should be active and not.
	 * <p>
	 * Can either be a function callback or a configuration object.
	 * <p>
	 * If a function is used, it should return a <code>boolean</code> for whether or not the track should be active.
	 * The function is called for each audio event, and receives a parameter object with <code>time</code>,
	 * and potentially <code>point</code> and <code>value</code> properties depending on the track.
	 * <code>point</code> is available if the audio event is related to a data point.
	 * <code>value</code> is available if the track is used as a context track, and <code>valueInterval</code> is used.
	 */
	public Object getActiveWhen() {
		Object activeWhen = getAttr(Attrs.activeWhen);
		if (activeWhen == null) {
			activeWhen = new ActiveWhen();
			setActiveWhen((ActiveWhen) activeWhen);
		}
		return activeWhen;
	}

	/**
	 * Sets the ActiveWhen object.
	 * <p>
	 * Define a condition for when a track should be active and not.
	 * <p>
	 * Can either be a function callback or a configuration object.
	 * <p>
	 * If a function is used, it should return a <code>boolean</code> for whether or not the track should be active.
	 * The function is called for each audio event, and receives a parameter object with <code>time</code>,
	 * and potentially <code>point</code> and <code>value</code> properties depending on the track.
	 * <code>point</code> is available if the audio event is related to a data point.
	 * <code>value</code> is available if the track is used as a context track, and <code>valueInterval</code> is used.
	 */
	public void setActiveWhen(ActiveWhen activeWhen) {
		setAttr(Attrs.activeWhen, activeWhen);
	}

	/**
	 * Sets the JavaScriptValue of active when.
	 * <p>
	 * Define a condition for when a track should be active and not.
	 * <p>
	 * Can either be a function callback or a configuration object.
	 * <p>
	 * If a function is used, it should return a <code>boolean</code> for whether or not the track should be active.
	 * The function is called for each audio event, and receives a parameter object with <code>time</code>,
	 * and potentially <code>point</code> and <code>value</code> properties depending on the track.
	 * <code>point</code> is available if the audio event is related to a data point.
	 * <code>value</code> is available if the track is used as a context track, and <code>valueInterval</code> is used.
	 */
	public void setActiveWhen(JavaScriptValue activeWhen) {
		setAttr(Attrs.activeWhen, activeWhen);
	}

	/**
	 * Returns instrument to use for playing.
	 * <p>Can either be a string referencing a synth preset, or it can be a synth configuration object.
	 * <p>Default: {@code "piano"}
	 */
	public String getInstrument() {
		return getAttr(Attrs.instrument, "piano").asString();
	}

	/**
	 * Sets instrument to use for playing.
	 * <p>Can either be a string referencing a synth preset, or it can be a synth configuration object.
	 */
	public void setInstrument(String instrument) {
		setAttr(Attrs.instrument, instrument, "piano");
	}

	/**
	 * Returns the Mapping object.
	 * <p>Mapping options for the audio parameters.
	 * <p>All parameters can be either:
	 * <ul>
	 *     <li>A string, referencing a point property to map to.</li>
	 *     <li>A number, setting the value of the audio parameter directly.</li>
	 *     <li>A callback function, returning the value programmatically.</li>
	 *     <li>An object defining detailed configuration of the mapping.</li>
	 * </ul>
	 * If a function is used, it should return the desired value for the audio parameter.
	 * The function is called for each audio event to be played,
	 * and receives a context object parameter with time,
	 * and potentially point and value depending on the track.
	 * point is available if the audio event is related to a data point,
	 * and value is available if the track is used for a context track using valueInterval.
	 */
	public Mapping getMapping() {
		Mapping mapping = (Mapping) getAttr(Attrs.mapping);
		if (mapping == null) {
			mapping = new Mapping();
			setMapping(mapping);
		}
		return mapping;
	}

	/**
	 * Sets the Mapping object.
	 * <p>Mapping options for the audio parameters.
	 * <p>All parameters can be either:
	 * <ul>
	 *     <li>A string, referencing a point property to map to.</li>
	 *     <li>A number, setting the value of the audio parameter directly.</li>
	 *     <li>A callback function, returning the value programmatically.</li>
	 *     <li>An object defining detailed configuration of the mapping.</li>
	 * </ul>
	 * If a function is used, it should return the desired value for the audio parameter.
	 * The function is called for each audio event to be played,
	 * and receives a context object parameter with time,
	 * and potentially point and value depending on the track.
	 * point is available if the audio event is related to a data point,
	 * and value is available if the track is used for a context track using valueInterval.
	 */
	public void setMapping(Mapping mapping) {
		setAttr(Attrs.mapping, mapping);
	}

	/**
	 * Returns the name to use for a track when exporting to MIDI.
	 * By default it uses the series name if the track is related to a series.
	 * Default: null
	 */
	public String getMidiName() {
		return getAttr(Attrs.midiName, null).asString();
	}

	/**
	 * Sets the name to use for a track when exporting to MIDI.
	 * By default it uses the series name if the track is related to a series.
	 */
	public void setMidiName(String midiName) {
		setAttr(Attrs.midiName, midiName, null);
	}

	/**
	 * Returns the options for point grouping, specifically for instrument tracks.
	 */
	public PointGrouping getPointGrouping() {
		PointGrouping pointGrouping = (PointGrouping) getAttr(Attrs.pointGrouping);
		if (pointGrouping == null) {
			pointGrouping = new PointGrouping();
			setPointGrouping(pointGrouping);
		}
		return pointGrouping;
	}

	/**
	 * Sets the options for point grouping, specifically for instrument tracks.
	 */
	public void setPointGrouping(PointGrouping pointGrouping) {
		setAttr(Attrs.pointGrouping, pointGrouping);
	}

	/**
	 * Returns the round pitch mapping to musical notes.
	 * <p>If false, will play the exact mapped note,
	 * even if it is out of tune compared to the musical notes as defined by 440Hz standard tuning.
	 * <p>Default: {@code true}
	 */
	public boolean isRoundToMusicalNotes() {
		return getAttr(Attrs.roundToMusicalNotes, true).asBoolean();
	}

	/**
	 * Sets the round pitch mapping to musical notes.
	 * <p>If false, will play the exact mapped note,
	 * even if it is out of tune compared to the musical notes as defined by 440Hz standard tuning.
	 */
	public void setRoundToMusicalNotes(boolean roundToMusicalNotes) {
		setAttr(Attrs.roundToMusicalNotes, roundToMusicalNotes, true);
	}

	/**
	 * Returns the round pitch mapping to musical notes.
	 * <p>If false, will play the exact mapped note,
	 * even if it is out of tune compared to the musical notes as defined by 440Hz standard tuning.
	 * <p>Default: {@code true}
	 */
	public boolean isShowPlayMarker() {
		return getAttr(Attrs.showPlayMarker, true).asBoolean();
	}

	/**
	 * Sets the round pitch mapping to musical notes.
	 * <p>If false, will play the exact mapped note,
	 * even if it is out of tune compared to the musical notes as defined by 440Hz standard tuning.
	 */
	public void setShowPlayMarker(boolean showPlayMarker) {
		setAttr(Attrs.showPlayMarker, showPlayMarker, true);
	}

	/**
	 * Returns the type of track. Always <code>"instrument"</code> for instrument tracks,
	 * and "speech" for <code>speech</code> tracks.
	 * <p>Default: {@code "instrument"}
	 */
	public String getType() {
		return getAttr(Attrs.type, "instrument").asString();
	}

	/**
	 * Sets the type of track. Always <code>"instrument"</code> for instrument tracks,
	 * and "speech" for <code>speech</code> tracks.
	 */
	public void setType(String type) {
		setAttr(Attrs.type, type, "instrument");
	}

}
