/* Pitch.java

        Purpose:
                
        Description:
                
        History:
                Mon Oct 16 14:10:06 CST 2023, Created by jamson

Copyright (C) 2023 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import java.util.Arrays;
import java.util.List;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.lang.Generics;

/**
 * Musical pitch refers to how high or low notes are played.
 * <p>By default it is mapped to y so low y values are played with a lower pitch, and high values are played with a higher pitch.
 * <p>Pitch mapping has a few extra features compared to other audio parameters.
 * <p>Firstly, it accepts not only number values, but also string values denoting note names. These are given in the form <note><octave>, for example "c6" or "F#2".
 * <p>Secondly, it is possible to map pitch to an array of notes. In this case, the [gapBetweenNotes](#sonification.defaultInstrumentOptions.mapping.gapBetweenNotes) mapping determines the delay between these notes.
 * <p>Thirdly, it is possible to define a musical scale to follow when mapping.
 * <p>Can be set to a fixed value, an array, a prop to map to, a function, or a mapping object.
 * @author Jamson Chan
 * @since 11.2.0.0
 */
public class Pitch extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		mapFunction,
		mapTo,
		max,
		min,
		scale,
		value,
		within
	}

	/**
	 * Returns how to perform the mapping. ("linear", "logarithmic")
	 * <p>
	 * Default: null
	 */
	public String getMapFunction() {
		return getAttr(Attrs.mapFunction, null).asString();
	}

	/**
	 * Sets how to perform the mapping. ("linear", "logarithmic")
	 */
	public void setMapFunction(String mapFunction) {
		setAttr(Attrs.mapFunction, mapFunction);
	}

	/**
	 * Returns a point property to map the mapping parameter to.
	 * <p>A negative sign - can be placed before the property name to make mapping inverted.
	 * <p>
	 * Default: "y"
	 */
	public String getMapTo() {
		return getAttr(Attrs.mapTo, "y").asString();
	}

	/**
	 * Sets a point property to map the mapping parameter to.
	 * <p>A negative sign - can be placed before the property name to make mapping inverted.
	 */
	public void setMapTo(String mapTo) {
		setAttr(Attrs.mapTo, mapTo);
	}

	/**
	 * Returns the maximum value for the audio parameter. This is the highest value the audio parameter will be mapped to.
	 * <p>
	 * Default: "c6"
	 */
	public String getMax() {
		return getAttr(Attrs.max, "c6").asString();
	}

	/**
	 * Sets the maximum value for the audio parameter. This is the highest value the audio parameter will be mapped to.
	 */
	public void setMax(String max) {
		setAttr(Attrs.max, max);
	}

	/**
	 * Returns the minimum value for the audio parameter. This is the lowest value the audio parameter will be mapped to.
	 * <p>
	 * Default: "c2"
	 */
	public String getMin() {
		return getAttr(Attrs.min, "c2").asString();
	}

	/**
	 * Sets the minimum value for the audio parameter. This is the lowest value the audio parameter will be mapped to.
	 */
	public void setMin(String min) {
		setAttr(Attrs.min, min);
	}

	/**
	 * Returns the scale array.
	 * <p>Map pitches to a musical scale.
	 * The scale is defined as an array of semitone offsets from the root note.
	 * <p>
	 * Default: null
	 */
	public List<Number> getScale() {
		return Generics.cast(getAttr(Attrs.scale, null).asValue());
	}

	/**
	 * Sets the scale array.
	 * <p>Map pitches to a musical scale.
	 * The scale is defined as an array of semitone offsets from the root note.
	 */
	public void setScale(List<Number> scale) {
		setAttr(Attrs.scale, scale);
	}

	/**
	 * Sets the scale array.
	 * <p>Map pitches to a musical scale.
	 * The scale is defined as an array of semitone offsets from the root note.
	 */
	public void setScale(Number... scale) {
		setAttr(Attrs.scale, Arrays.asList(scale));
	}

	/**
	 * Returns a fixed value to use for the prop when mapping.
	 * <p>For example, if mapping to y, setting value to 4 will map as if all points had a y value of 4.
	 * <p>
	 * Default: null
	 */
	public Number getValue() {
		return getAttr(Attrs.value, null).asNumber();
	}

	/**
	 * Sets a fixed value to use for the prop when mapping.
	 * <p>For example, if mapping to y, setting value to 4 will map as if all points had a y value of 4.
	 */
	public void setValue(Number value) {
		setAttr(Attrs.value, value);
	}

	/**
	 * Returns what data values to map the parameter within.
	 * <p>Mapping within "series" will make the lowest value point in the series map to the min audio parameter value, and the highest value will map to the max audio parameter.
	 * <p>Mapping within "chart" will make the lowest value point in the whole chart map to the min audio parameter value, and the highest value in the whole chart will map to the max audio parameter.
	 * <p>You can also map within the X or Y axis of each series.
	 * <p>
	 * Default: null
	 */
	public String getWithin() {
		return getAttr(Attrs.within, null).asString();
	}

	/**
	 * Sets what data values to map the parameter within.
	 * <p>Mapping within "series" will make the lowest value point in the series map to the min audio parameter value, and the highest value will map to the max audio parameter.
	 * <p>Mapping within "chart" will make the lowest value point in the whole chart map to the min audio parameter value, and the highest value in the whole chart will map to the max audio parameter.
	 * <p>You can also map within the X or Y axis of each series.
	 */
	public void setWithin(String within) {
		setAttr(Attrs.within, within);
	}
}
