/* PlotLabel.java

	Purpose:
		
	Description:
		
	History:
		Fri, Jan 17, 2014 12:03:02 PM, Created by RaymondChao

Copyright (C) 2014 Potix Corporation. All Rights Reserved.

 */
package org.zkoss.chart;

import java.util.LinkedHashMap;
import java.util.Map;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.chart.util.MapsHelper;
import org.zkoss.lang.Generics;

/**
 * Text labels for the plot bands and plot lines
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 * 
 * @author RaymondChao
 * @author jumperchen
 */
public class PlotLabel extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		align, rotation, style, text, textAlign, useHTML, verticalAlign, x, y
	}

	/**
	 * Returns horizontal alignment of the label. Can be one of "left", "center"
	 * or "right".
	 * <p>
	 * Default: "center"
	 */
	public String getAlign() {
		return getAttr(Attrs.align, "center").asString();
	}

	/**
	 * Sets horizontal alignment of the label. Can be one of "left", "center" or
	 * "right".
	 */
	public void setAlign(String align) {
		if (!("left".equals(align) || "center".equals(align) || "right"
				.equals(align)))
			throw new IllegalArgumentException("Unsupported align: [" + align
					+ "]");
		setAttr(Attrs.align, align, NOT_NULL_VALUE);
	}

	/**
	 * Returns rotation of the text label in degrees.
	 * <p>
	 * Default: 0
	 */
	public Number getRotation() {
		return getAttr(Attrs.rotation, 0).asNumber();
	}

	/**
	 * Sets rotation of the text label in degrees . 'rotation rotation of the
	 * text label in degrees.
	 */
	public void setRotation(Number rotation) {
		setAttr(Attrs.rotation, rotation);
	}

	/**
	 * Returns CSS styles for the text label.
	 * @param <K> type for the key
	 * <p>
	 * if style was set using {@link #setStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, type will be Object by default
	 * @param <V> type for the value
	 * <p>
	 * if style was set using {@link #setStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, type will be Object by default
	 */
	public <K, V> Map<K, V> getStyle() {
		return Generics.cast(getAttr(Attrs.style, null).asValue());
	}

	/**
	 * Sets CSS styles for the text label.
	 * <p>
	 * This method allows <b>simple</b> settings like
	 * <pre>{@code
	 * label.setStyle("fontWeight: 'bold'; color: 'white'; textShadow: '0px 1px 2px black'");
	 * }</pre>
	 * <p>
	 * In the format of CSS key:value, and ";" separating multiple key:value pairs,
	 * if you have more a complex CSS styling, we recommend using {@link #setStyle(Map)}
	 * @param style one or many CSS key:value pairs in a single line string format
	 */
	public void setStyle(String style) {
		setStyle(MapsHelper.parse(new LinkedHashMap<String, String>(), style, ':', ';', '\''));
	}

	/**
	 * Sets CSS styles for the text label.
	 * <p>
	 * This method allows <b>complex</b> styling like:
	 * <pre>
	 * {@code
	 * Map<String, Object> styles = new HashMap<String, Object>();
	 * styles.put("fill", "#E8E8EA");
	 * styles.put("stroke", "#6E6E78");
	 * styles.put("stroke-width", "1");
	 * styles.put("r", "0");
	 * Map<String, Object> states = new HashMap<String, Object>();
	 * Map<String, String> hover = new HashMap<String, String>();
	 * hover.put("fill", "#F4F4F6");
	 * hover.put("stroke", "#6E6E78");
	 * hover.put("stroke-width", "1");
	 * hover.put("r", "0");
	 * states.put("hover", hover);
	 * styles.put("states", states);
	 * Map<String, String> style = new HashMap<String, String>();
	 * style.put("color", "#4D5663");
	 * style.put("fontSize", "12px");
	 * style.put("fontFamily", "'Arial'");
	 * style.put("fontWeight", "normal");
	 * style.put("padding", "10px");
	 * styles.put("style", style);
	 * }</pre>
	 * which will generate the final results like:
	 * <pre>{@code
	 * {
	 *     "fill": "#E8E8EA",
	 *     "stroke": "#6E6E78",
	 *     "stroke-width": "1",
	 *     "r": "0",
	 *     "states":
	 *     {
	 *         "hover":
	 *         {
	 *             "fill": "#F4F4F6",
	 *             "stroke": "#6E6E78",
	 *             "stroke-width": "1",
	 *             "r": "0"
	 *         }
	 *     },
	 *     "style":
	 *     {
	 *         "color": "#4D5663",
	 *         "fontSize": "12px",
	 *         "fontFamily": "'Arial'",
	 *         "fontWeight": "normal",
	 *         "padding": "10px"
	 *     }
	 * }
	 * }</pre>
	 * @param <K> Type for the key, usually String
	 * @param <V> Type for the value, usually String. For nested maps, use Object or any other type.
	 * @param style the style object
	 */
	public <K, V> void setStyle(Map<K, V> style) {
		setAttr(Attrs.style, style);
	}

	/**
	 * Returns the string text itself. A subset of HTML is supported.
	 */
	public String getText() {
		return getAttr(Attrs.text, null).asString();
	}

	/**
	 * Sets the string text itself. A subset of HTML is supported.
	 */
	public void setText(String text) {
		setAttr(Attrs.text, text);
	}

	/**
	 * Returns the text alignment for the label. While <code>align</code>
	 * determines where the texts anchor point is placed within the plot band,
	 * <code>textAlign</code> determines how the text is aligned against its
	 * anchor point. Possible values are "left", "center" and "right". Defaults
	 * to the same as the <code>align</code> option.
	 */
	public String getTextAlign() {
		if (containsKey(Attrs.textAlign)) {
			return (String) getAttr(Attrs.textAlign);
		} else {
			return getAlign();
		}
	}

	/**
	 * Sets the text alignment for the label. While <code>align</code>
	 * determines where the texts anchor point is placed within the plot band,
	 * <code>textAlign</code> determines how the text is aligned against its
	 * anchor point. Possible values are "left", "center" and "right". Defaults
	 * to the same as the <code>align</code> option.
	 */
	public void setTextAlign(String textAlign) {
		if (!("left".equals(textAlign) || "center".equals(textAlign) || "right"
				.equals(textAlign)))
			throw new IllegalArgumentException("Unsupported textAlign: ["
					+ textAlign + "]");
		setAttr(Attrs.textAlign, textAlign);
	}

	/**
	 * Returns whether to <a
	 * href="http://docs.highcharts.com/#formatting$html">use HTML</a> to render
	 * the labels.
	 * <p>
	 * Default: false
	 */
	public boolean isUseHTML() {
		return getAttr(Attrs.useHTML, false).asBoolean();
	}

	/**
	 * Sets whether to <a href="http://docs.highcharts.com/#formatting$html">use
	 * HTML</a> to render the labels.
	 * <p>
	 * Default: false
	 */
	public void setUseHTML(boolean useHTML) {
		setAttr(Attrs.useHTML, useHTML);
	}

	/**
	 * Returns vertical alignment of the label relative to the plot band. Can be
	 * one of "top", "middle" or "bottom".
	 * <p>
	 * Default: "top"
	 */
	public String getVerticalAlign() {
		return getAttr(Attrs.verticalAlign, "top").asString();
	}

	/**
	 * Sets vertical alignment of the label relative to the plot band. Can be
	 * one of "top", "middle" or "bottom".
	 */
	public void setVerticalAlign(String verticalAlign) {
		if (!("top".equals(verticalAlign) || "middle".equals(verticalAlign) || "bottom"
				.equals(verticalAlign)))
			throw new IllegalArgumentException("Unsupported verticalAlign: ["
					+ verticalAlign + "]");
		setAttr(Attrs.verticalAlign, verticalAlign, "top");
	}

	/**
	 * Returns horizontal position relative the alignment.
	 * <p>
	 * Default varies by orientation.
	 */
	public Number getX() {
		return getAttr(Attrs.x, null).asNumber();
	}

	/**
	 * Sets horizontal position relative the alignment. Default varies by
	 * orientation.
	 */
	public void setX(Number x) {
		setAttr(Attrs.x, x);
	}

	/**
	 * Returns vertical position of the text baseline relative to the alignment.
	 * <p>
	 * Default varies by orientation.
	 */
	public Number getY() {
		return getAttr(Attrs.y, null).asNumber();
	}

	/**
	 * Sets vertical position of the text baseline relative to the alignment.
	 * Default varies by orientation.
	 */
	public void setY(Number y) {
		setAttr(Attrs.y, y);
	}

}