/* ScrollablePlotArea.java

	Purpose:

	Description:

	History:
		Thu Jun 24 17:20:01 CST 2021, Created by apicard

Copyright (C) 2021 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import org.zkoss.chart.util.DynamicalAttribute;

/**
 * Options for a scrollable plot area. This feature provides a minimum size for the plot area of the chart. If
 * the size gets smaller than this, typically on mobile devices, a native browser scrollbar is presented. This
 * scrollbar provides smooth scrolling for the contents of the plot area, whereas the title, legend and
 * unaffected axes are fixed.
 * 
 * Since v7.1.2, a scrollable plot area can be defined for either horizontal or vertical scrolling, depending
 * on whether the minWidth or minHeight option is set.
 * 
 * @author apicard
 * @since 7.2.2.0
 */
public class ScrollablePlotArea extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		minHeight,
		minWidth, 
		opacity, 
		scrollPositionX,
		scrollPositionY
	}

	/**
	 * Returns the minimum height for the plot area. If it gets smaller than this, the plot area will become scrollable.
	 * <p>
	 * Default: undefined
	 */
	public Number getMinHeight() {
		return getAttr(Attrs.minHeight, 0).asNumber();
	}

	/**
	 * Sets the minimum height for the plot area. If it gets smaller than this, the plot area will become scrollable.
	 */
	public void setMinHeight(Number minHeight) {
		setAttr(Attrs.minHeight, minHeight);
	}

	/**
	 * Returns the minimum width for the plot area. If it gets smaller than this, the plot area will become scrollable.
	 * <p>
	 * Default: undefined
	 */
	public Number getMinWidth() {
		return getAttr(Attrs.minWidth, 0).asNumber();
	}

	/**
	 * Sets the minimum width for the plot area. If it gets smaller than this, the plot area will become scrollable.
	 */
	public void setMinWidth(Number minWidth) {
		setAttr(Attrs.minWidth, minWidth);
	}

	/**
	 * Returns the opacity of mask applied on one of the sides of the plot area.
	 * <p>
	 * Default: 0.85.
	 */
	public Number getOpacity() {
		return getAttr(Attrs.opacity, 0.85).asNumber();
	}

	/**
	 * Sets the opacity of mask applied on one of the sides of the plot area.
	 * 
	 */
	public void setOpacity(Number opacity) {
		setAttr(Attrs.opacity, opacity, 0.85);
	}

	/**
	 * Returns the initial scrolling position of the scrollable plot area. Ranges from 0 to 1, where 0 aligns
	 * the plot area to the left and 1 aligns it to the right. Typically we would use 1 if the chart has right
	 * aligned Y axes.
	 * <p>
	 * Default: undefined
	 */
	public Number getScrollPositionX() {
		return getAttr(Attrs.scrollPositionX, 0).asNumber();
	}

	/**
	 * Sets the initial scrolling position of the scrollable plot area. Ranges from 0 to 1, where 0 aligns
	 * the plot area to the left and 1 aligns it to the right. Typically we would use 1 if the chart has right
	 * aligned Y axes.
	 */
	public void setScrollPositionX(Number scrollPositionX) {
		setAttr(Attrs.scrollPositionX, scrollPositionX);
	}

	/**
	 * Returns the initial scrolling position of the scrollable plot area. Ranges from 0 to 1, where 0 aligns
	 * the plot area to the top and 1 aligns it to the bottom.
	 * <p>
	 * Default: undefined
	 */
	public Number getScrollPositionY() {
		return getAttr(Attrs.scrollPositionY, 0).asNumber();
	}

	/**
	 * Sets the initial scrolling position of the scrollable plot area. Ranges from 0 to 1, where 0 aligns
	 * the plot area to the top and 1 aligns it to the bottom.
	 */
	public void setScrollPositionY(Number scrollPositionY) {
		setAttr(Attrs.scrollPositionY, scrollPositionY);
	}
}