/* Sonification.java

        Purpose:
                
        Description:
                
        History:
                Fri Sep 15 13:00:33 CST 2023, Created by jamson

Copyright (C) 2023 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import java.util.LinkedList;
import java.util.List;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.json.JavaScriptValue;
import org.zkoss.lang.Generics;

/**
 * Options for configuring sonification and audio charts. Requires the sonification module to be loaded.
 * @author Jamson Chan
 * @since 11.2.0.0
 */
public class Sonification extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		afterSeriesWait,
		defaultInstrumentOptions,
		defaultSpeechOptions,
		duration,
		enabled,
		events,
		globalContextTracks,
		globalTracks,
		masterVolume,
		order,
		pointGrouping,
		showCrosshair,
		showTooltip,
		updateInterval,
	}

	/**
	 * Returns the time to wait in milliseconds after each data series when playing the series one after the other.
	 * <p>Default: {@code 700}
	 */
	public Number getAfterSeriesWait() {
		return getAttr(Attrs.afterSeriesWait, 700).asNumber();
	}

	/**
	 * Sets the time to wait in milliseconds after each data series when playing the series one after the other.
	 */
	public void setAfterSeriesWait(Number afterSeriesWait) {
		setAttr(Attrs.afterSeriesWait, afterSeriesWait, 700);
	}

	/**
	 * Returns the default sonification options for all instrument tracks.
	 * <p>If specific options are also set on individual tracks or per series, those will override these options.
	 */
	public DefaultInstrumentOptions getDefaultInstrumentOptions() {
		DefaultInstrumentOptions defaultInstrumentOptions = (DefaultInstrumentOptions) getAttr(Attrs.defaultInstrumentOptions);
		if (defaultInstrumentOptions == null) {
			defaultInstrumentOptions = new DefaultInstrumentOptions();
			setDefaultInstrumentOptions(defaultInstrumentOptions);
		}
		return defaultInstrumentOptions;
	}

	/**
	 * Sets the default sonification options for all instrument tracks.
	 * <p>If specific options are also set on individual tracks or per series, those will override these options.
	 */
	public void setDefaultInstrumentOptions(DefaultInstrumentOptions defaultInstrumentOptions) {
		setAttr(Attrs.defaultInstrumentOptions, defaultInstrumentOptions);
	}

	/**
	 * Returns the default sonification options for all speech tracks.
	 */
	public DefaultSpeechOptions getDefaultSpeechOptions() {
		DefaultSpeechOptions defaultSpeechOptions = (DefaultSpeechOptions) getAttr(Attrs.defaultSpeechOptions);
		if (defaultSpeechOptions == null) {
			defaultSpeechOptions = new DefaultSpeechOptions();
			setDefaultSpeechOptions(defaultSpeechOptions);
		}
		return defaultSpeechOptions;
	}

	/**
	 * Sets the default sonification options for all speech tracks.
	 */
	public void setDefaultSpeechOptions(DefaultSpeechOptions defaultSpeechOptions) {
		setAttr(Attrs.defaultSpeechOptions, defaultSpeechOptions);
	}

	/**
	 * Returns the total duration of the sonification, in milliseconds.
	 * <p>Default: {@code 6000}
	 */
	public Number getDuration() {
		return getAttr(Attrs.duration, 6000).asNumber();
	}

	/**
	 * Sets the total duration of the sonification, in milliseconds.
	 */
	public void setDuration(Number duration) {
		setAttr(Attrs.duration, duration, 6000);
	}

	/**
	 * Returns whether to enable or disable the sonification functionality for the chart.
	 * <p>Default: {@code true}
	 */
	public boolean isEnabled() {
		return getAttr(Attrs.enabled, true).asBoolean();
	}

	/**
	 * Sets whether to enable or disable the sonification functionality for the chart.
	 */
	public void setEnabled(boolean enabled) {
		setAttr(Attrs.enabled, enabled, true);
	}

	/**
	 * Returns the event handlers for the sonification.
	 * <p>Default: {@code null}
	 */
	public JavaScriptValue getEvents() {
		return (JavaScriptValue) getAttr(Attrs.events, null).asValue();
	}

	/**
	 * Sets the event handlers for the sonification.
	 */
	public void setEvents(JavaScriptValue events) {
		setAttr(Attrs.events, events);
	}

	/**
	 * Returns the size of globalContextTracks.
	 */
	public int getGlobalContextTracksSize() {
		List<Series> list = Generics.cast(getAttr(Attrs.globalContextTracks));
		return list == null ? 0 : list.size();
	}

	/**
	 * Returns the GlobalContextTrack options, if any, Otherwise, create a new one.
	 * <p>
	 * Returns the first node (index 0) by default.
	 * <p>Context tracks to add globally, an array of either instrument tracks, speech tracks, or a mix.
	 * <p>Context tracks are not tied to data points, but play at a set interval - either based on time or on prop values.
	 */
	public GlobalContextTrack getGlobalContextTracks() {
		return getGlobalContextTracks(0);
	}

	/**
	 * Returns the GlobalContextTrack options from given index, if any, Otherwise,
	 * create a new one.
	 * <p>Context tracks to add globally, an array of either instrument tracks, speech tracks, or a mix.
	 * <p>Context tracks are not tied to data points, but play at a set interval - either based on time or on prop values.
	 */
	public GlobalContextTrack getGlobalContextTracks(int index) {
		List<GlobalContextTrack> list = Generics.cast(getAttr(Attrs.globalContextTracks));
		if (list == null) {
			list = new LinkedList<GlobalContextTrack>();
			setAttr(Attrs.globalContextTracks, list);
		}

		int size = list.size();
		if (size <= index) {
			for (int i = index - size + 1; i > 0; i--) {
				list.add(new GlobalContextTrack());
			}
		}
		return list.get(index);
	}




	/**
	 * Returns the size of globalTracks.
	 */
	public int getGlobalTracksSize() {
		List<Series> list = Generics.cast(getAttr(Attrs.globalTracks));
		return list == null ? 0 : list.size();
	}

	/**
	 * Returns the GlobatTrack options, if any, Otherwise, create a new one.
	 * <p>
	 * Returns the first node (index 0) by default.
	 * <p>Global tracks to add to every series.
	 * <p>Defined as an array of either instrument or speech tracks, or a combination.
	 */
	public GlobalTrack getGlobalTracks() {
		return getGlobalTracks(0);
	}

	/**
	 * Returns the GlobatTrack options from given index, if any, Otherwise,
	 * create a new one.
	 * <p>Global tracks to add to every series.
	 * <p>Defined as an array of either instrument or speech tracks, or a combination.
	 */
	public GlobalTrack getGlobalTracks(int index) {
		List<GlobalTrack> list = Generics.cast(getAttr(Attrs.globalTracks));
		if (list == null) {
			list = new LinkedList<GlobalTrack>();
			setAttr(Attrs.globalTracks, list);
		}

		int size = list.size();
		if (size <= index) {
			for (int i = index - size + 1; i > 0; i--) {
				list.add(new GlobalTrack());
			}
		}
		return list.get(index);
	}

	/**
	 * Returns overall/master volume for the sonification, from 0 to 1.
	 * <p>Default: {@code 0.7}
	 */
	public Number getMasterVolume() {
		return getAttr(Attrs.masterVolume, 0.7).asNumber();
	}

	/**
	 * Sets overall/master volume for the sonification, from 0 to 1.
	 */
	public void setMasterVolume(Number masterVolume) {
		setAttr(Attrs.masterVolume, masterVolume, 0.7);
	}

	/**
	 * Returns what order to play the data series in,
	 * either sequential where the series play individually one after the other,
	 * or simultaneous where the series play all at once.
	 * <p>Default: {@code "sequential"}
	 */
	public String getOrder() {
		return getAttr(Attrs.order, "sequential").asString();
	}

	/**
	 * Sets what order to play the data series in,
	 * either sequential where the series play individually one after the other,
	 * or simultaneous where the series play all at once.
	 */
	public void setOrder(String order) {
		setAttr(Attrs.order, order, "sequential");
	}

	/**
	 * Returns the PointGrouping object.
	 * Options for grouping data points together when sonifying.
	 * This allows for the visual presentation to contain more points than what is being played.
	 * If not enabled, all visible / uncropped points are played.
	 */
	public PointGrouping getPointGrouping() {
		PointGrouping pointGrouping = (PointGrouping) getAttr(Attrs.pointGrouping);
		if (pointGrouping == null) {
			pointGrouping = new PointGrouping();
			setPointGrouping(pointGrouping);
		}
		return pointGrouping;
	}

	/**
	 * Sets the PointGrouping object.
	 * Options for grouping data points together when sonifying.
	 * This allows for the visual presentation to contain more points than what is being played.
	 * If not enabled, all visible / uncropped points are played.
	 */
	public void setPointGrouping(PointGrouping pointGrouping) {
		setAttr(Attrs.pointGrouping, pointGrouping);
	}

	/**
	 * Returns whether to show X and Y axis crosshairs (if they exist) as the chart plays.
	 * Note that if multiple tracks that play at different times try to show the crosshairs,
	 * it can be glitchy, so it is recommended in those cases to turn this on/off for individual tracks using the showPlayMarker option.
	 * <p>Default: {@code true}
	 */
	public boolean isShowCrosshair() {
		return getAttr(Attrs.showCrosshair, true).asBoolean();
	}

	/**
	 * Sets whether to show X and Y axis crosshairs (if they exist) as the chart plays.
	 * Note that if multiple tracks that play at different times try to show the crosshairs,
	 * it can be glitchy, so it is recommended in those cases to turn this on/off for individual tracks using the showPlayMarker option.
	 */
	public void setShowCrosshair(boolean showCrosshair) {
		setAttr(Attrs.showCrosshair, showCrosshair, true);
	}

	/**
	 * Returns whether to Show tooltip as the chart plays.
	 * Note that if multiple tracks that play at different times try to show the tooltip,
	 * it can be glitchy, so it is recommended in those cases to turn this on/off for individual tracks using the showPlayMarker option.
	 * <p>Default: {@code true}
	 */
	public boolean isShowTooltip() {
		return getAttr(Attrs.showTooltip, true).asBoolean();
	}

	/**
	 * Sets whether to Show tooltip as the chart plays.
	 * Note that if multiple tracks that play at different times try to show the tooltip,
	 * it can be glitchy, so it is recommended in those cases to turn this on/off for individual tracks using the showPlayMarker option.
	 */
	public void setShowTooltip(boolean showTooltip) {
		setAttr(Attrs.showTooltip, showTooltip, true);
	}

	/**
	 * Returns how long to wait between each recomputation of the sonification,
	 * if the chart updates rapidly. This avoids slowing down processes like panning.Given in milliseconds.
	 * <p>Default: {@code 200}
	 */
	public Number getUpdateInterval() {
		return getAttr(Attrs.updateInterval, 200).asNumber();
	}

	/**
	 * Sets how long to wait between each recomputation of the sonification,
	 * if the chart updates rapidly. This avoids slowing down processes like panning.Given in milliseconds.
	 */
	public void setUpdateInterval(Number updateInterval) {
		setAttr(Attrs.updateInterval, updateInterval, 200);
	}
}
