/* BoxPlotOptions.java

	Purpose:
		
	Description:
		
	History:
		Thu, Jan 16, 2014  3:55:39 PM, Created by RaymondChao

Copyright (C) 2014 Potix Corporation. All Rights Reserved.

 */
package org.zkoss.chart.plotOptions;

import org.zkoss.chart.Color;
import org.zkoss.chart.LinearGradient;
import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.RadialGradient;
import org.zkoss.chart.Series;
import org.zkoss.chart.util.DynamicalAttribute;

/**
 * An option for box plot
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 * 
 * @author RaymondChao
 */
public class BoxPlotOptions extends ColumnPlotOptions {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		boxDashStyle,
		fillColor,
		lineWidth,
		medianColor,
		medianDashStyle,
		medianWidth,
		stemColor,
		stemDashStyle,
		stemWidth,
		whiskerColor,
		whiskerDashStyle,
		whiskerLength,
		whiskerWidth
	}

	/**
	 * Returns the dash style of the box.
	 *
	 * <p>Defaults to Solid.
	 * @since 10.2.1.
	 */
	public String getBoxDashStyle() {
		return getAttr(Attrs.boxDashStyle, "Solid").asString();
	}

	/**
	 * Sets the dash style of the box.
	 *
	 * <p>Defaults to Solid.
	 * @since 10.2.1.
	 */
	public void setBoxDashStyle(String boxDashStyle) {
		if (!("Solid".equalsIgnoreCase(boxDashStyle)
				|| "ShortDash".equalsIgnoreCase(boxDashStyle)
				|| "ShortDot".equalsIgnoreCase(boxDashStyle)
				|| "ShortDashDot".equalsIgnoreCase(boxDashStyle)
				|| "ShortDashDotDot".equalsIgnoreCase(boxDashStyle)
				|| "Dot".equalsIgnoreCase(boxDashStyle)
				|| "Dash".equalsIgnoreCase(boxDashStyle)
				|| "LongDash".equalsIgnoreCase(boxDashStyle)
				|| "DashDot".equalsIgnoreCase(boxDashStyle)
				|| "LongDashDot".equalsIgnoreCase(boxDashStyle) || "LongDashDotDot"
				.equalsIgnoreCase(boxDashStyle)))
			throw new IllegalArgumentException("Unsupported style: ["
					+ boxDashStyle + "]");
		setAttr(Attrs.boxDashStyle, boxDashStyle);
	}

	/**
	 * Returns the fill color of the box.
	 * <p>
	 * Default: #FFFFFF
	 */
	public Color getFillColor() {
		if (!containsKey(Attrs.fillColor)) {
			setFillColor("#FFFFFF");
		}
		return (Color) getAttr(Attrs.fillColor);
	}

	/**
	 * Sets the fill color of the box.
	 */
	public void setFillColor(Color color) {
		setAttr(Attrs.fillColor, color, NOT_NULL_VALUE);
	}

	/**
	 * Sets the fill color of the box.
	 */
	public void setFillColor(String color) {
		setFillColor(new Color(color));
	}

	/**
	 * Sets the fill color of the box.
	 */
	public void setFillColor(LinearGradient color) {
		setFillColor(new Color(color));
	}

	/**
	 * Sets the fill color of the box.
	 */
	public void setFillColor(RadialGradient color) {
		setFillColor(new Color(color));
	}

	/**
	 * Returns the width of the line surrounding the box. If any of
	 * {@link #setStemWidth(Number)} , {@link #setMedianWidth(Number)}, or
	 * {@link #setWhiskerWidth(Number)} are <code>null</code>, the lineWidth
	 * also applies to these lines.
	 */
	public Number getLineWidth() {
		return getAttr(Attrs.lineWidth, 1).asNumber();
	}

	/**
	 * Sets the width of the line surrounding the box. If any of
	 * {@link #setStemWidth(Number)} , {@link #setMedianWidth(Number)}, or
	 * {@link #setWhiskerWidth(Number)} are <code>null</code>, the lineWidth
	 * also applies to these lines.
	 */
	public void setLineWidth(Number lineWidth) {
		setAttr(Attrs.lineWidth, lineWidth, 1);
	}

	/**
	 * Returns the color of the median line. If <code>null</code>, the general
	 * series color applies.
	 */
	public String getMedianColor() {
		return getAttr(Attrs.medianColor, null).asString();
	}

	/**
	 * Sets the color of the median line. If <code>null</code>, the general
	 * series color applies. 'medianColor the color of the median line. If
	 * <code>null</code>, the general series color applies.
	 */
	public void setMedianColor(String medianColor) {
		setMedianColor(new Color(medianColor));
	}

	/**
	 * Sets the color of the median line. If <code>null</code>, the general
	 * series color applies. 'medianColor the color of the median line. If
	 * <code>null</code>, the general series color applies.
	 * @since 10.2.1.0
	 */
	public void setMedianColor(Color medianColor) {
		setAttr(Attrs.medianColor, medianColor);
	}

	/**
	 * Sets the color of the median line. If <code>null</code>, the general
	 * series color applies. 'medianColor the color of the median line. If
	 * <code>null</code>, the general series color applies.
	 * @since 10.2.1.0
	 */
	public void setMedianColor(LinearGradient medianColor) {
		setMedianColor(new Color(medianColor));
	}

	/**
	 * Sets the color of the median line. If <code>null</code>, the general
	 * series color applies. 'medianColor the color of the median line. If
	 * <code>null</code>, the general series color applies.
	 * @since 10.2.1.0
	 */
	public void setMedianColor(RadialGradient medianColor) {
		setMedianColor(new Color(medianColor));
	}

	/**
	 * Returns a name for the dash style of the median.
	 * The value for the <code>medianDashStyle</code> include:
	 * <ul>
	 * <li>Solid</li>
	 * <li>ShortDash</li>
	 * <li>ShortDot</li>
	 * <li>ShortDashDot</li>
	 * <li>ShortDashDotDot</li>
	 * <li>Dot</li>
	 * <li>Dash</li>
	 * <li>LongDash</li>
	 * <li>DashDot</li>
	 * <li>LongDashDot</li>
	 * <li>LongDashDotDot</li>
	 * </ul>
	 * <p>Default: {@code "Solid"}
	 * @since 10.2.1.0
	 */
	public String getMedianDashStyle() {
		return getAttr(Attrs.medianDashStyle, "Solid").asString();
	}

	/**
	 * Sets a name for the dash style of the median.
	 * The value for the <code>medianDashStyle</code> include:
	 * <ul>
	 * <li>Solid</li>
	 * <li>ShortDash</li>
	 * <li>ShortDot</li>
	 * <li>ShortDashDot</li>
	 * <li>ShortDashDotDot</li>
	 * <li>Dot</li>
	 * <li>Dash</li>
	 * <li>LongDash</li>
	 * <li>DashDot</li>
	 * <li>LongDashDot</li>
	 * <li>LongDashDotDot</li>
	 * </ul>
	 * @since 10.2.1.0
	 */
	public void setMedianDashStyle(String medianDashStyle) {
		if (!("Solid".equalsIgnoreCase(medianDashStyle)
				|| "ShortDash".equalsIgnoreCase(medianDashStyle)
				|| "ShortDot".equalsIgnoreCase(medianDashStyle)
				|| "ShortDashDot".equalsIgnoreCase(medianDashStyle)
				|| "ShortDashDotDot".equalsIgnoreCase(medianDashStyle)
				|| "Dot".equalsIgnoreCase(medianDashStyle)
				|| "Dash".equalsIgnoreCase(medianDashStyle)
				|| "LongDash".equalsIgnoreCase(medianDashStyle)
				|| "DashDot".equalsIgnoreCase(medianDashStyle)
				|| "LongDashDot".equalsIgnoreCase(medianDashStyle) || "LongDashDotDot"
					.equalsIgnoreCase(medianDashStyle)))
			throw new IllegalArgumentException("Unsupported style: ["
					+ medianDashStyle + "]");
		setAttr(Attrs.medianDashStyle, medianDashStyle);
	}

	/**
	 * Returns the pixel width of the median line. If <code>null</code>, the
	 * {@link #setLineWidth(Number)} is used.
	 * <p>
	 * Default: 2
	 */
	public Number getMedianWidth() {
		return getAttr(Attrs.medianWidth, 2).asNumber();
	}

	/**
	 * Sets the pixel width of the median line. If <code>null</code>, the
	 * {@link #setLineWidth(Number)} is used.
	 * <p>
	 * Default: 2
	 */
	public void setMedianWidth(Number medianWidth) {
		setAttr(Attrs.medianWidth, medianWidth, 2);
	}

	/**
	 * Returns the color of the stem, the vertical line extending from the box
	 * to the whiskers. If <code>null</code>, the series color is used.
	 */
	public Color getStemColor() {
		return (Color) getAttr(Attrs.stemColor, null).asValue();
	}

	/**
	 * Sets the color of the stem, the vertical line extending from the box to
	 * the whiskers. If <code>null</code>, the series color is used.
	 */
	public void setStemColor(Color color) {
		setAttr(Attrs.stemColor, color);
	}

	/**
	 * Sets the color of the stem, the vertical line extending from the box to
	 * the whiskers. If <code>null</code>, the series color is used.
	 */
	public void setStemColor(String color) {
		setStemColor(new Color(color));
	}

	/**
	 * Sets the color of the stem, the vertical line extending from the box to
	 * the whiskers. If <code>null</code>, the series color is used.
	 */
	public void setStemColor(LinearGradient color) {
		setStemColor(new Color(color));
	}

	/**
	 * Sets the color of the stem, the vertical line extending from the box to
	 * the whiskers. If <code>null</code>, the series color is used.
	 */
	public void setStemColor(RadialGradient color) {
		setStemColor(new Color(color));
	}

	/**
	 * Returns the dash style of the stem, the vertical line extending from the
	 * box to the whiskers.
	 */
	public String getStemDashStyle() {
		return getAttr(Attrs.stemDashStyle, "Solid").asString();
	}

	/**
	 * Sets the dash style of the stem, the vertical line extending from the box
	 * to the whiskers.
	 */
	public void setStemDashStyle(String stemDashStyle) {
		if (!("Solid".equalsIgnoreCase(stemDashStyle)
				|| "ShortDash".equalsIgnoreCase(stemDashStyle)
				|| "ShortDot".equalsIgnoreCase(stemDashStyle)
				|| "ShortDashDot".equalsIgnoreCase(stemDashStyle)
				|| "ShortDashDotDot".equalsIgnoreCase(stemDashStyle)
				|| "Dot".equalsIgnoreCase(stemDashStyle)
				|| "Dash".equalsIgnoreCase(stemDashStyle)
				|| "LongDash".equalsIgnoreCase(stemDashStyle)
				|| "DashDot".equalsIgnoreCase(stemDashStyle)
				|| "LongDashDot".equalsIgnoreCase(stemDashStyle) || "LongDashDotDot"
					.equalsIgnoreCase(stemDashStyle)))
			throw new IllegalArgumentException("Unsupported style: ["
					+ stemDashStyle + "]");
		setAttr(Attrs.stemDashStyle, stemDashStyle, "Solid");
	}

	/**
	 * Returns the width of the stem, the vertical line extending from the box
	 * to the whiskers. If <code>null</code>, the width is inherited from the
	 * {@link #setLineWidth(Number)} option.
	 */
	public Number getStemWidth() {
		return getAttr(Attrs.stemWidth, null).asNumber();
	}

	/**
	 * Sets the width of the stem, the vertical line extending from the box to
	 * the whiskers. If <code>null</code>, the width is inherited from the
	 * {@link #setLineWidth(Number)} option.
	 */
	public void setStemWidth(Number stemWidth) {
		setAttr(Attrs.stemWidth, stemWidth);
	}

	/**
	 * Returns the color of the whiskers, the horizontal lines marking low and
	 * high values. When <code>null</code>, the general series color is used.
	 * <p>
	 * Default: null
	 */
	public Color getWhiskerColor() {
		return (Color) getAttr(Attrs.whiskerColor, null).asValue();
	}

	/**
	 * Sets the color of the whiskers, the horizontal lines marking low and high
	 * values. When <code>null</code>, the general series color is used.
	 */
	public void setWhiskerColor(Color color) {
		setAttr(Attrs.whiskerColor, color);
	}

	/**
	 * Sets the color of the whiskers, the horizontal lines marking low and high
	 * values. When <code>null</code>, the general series color is used.
	 */
	public void setWhiskerColor(String color) {
		setWhiskerColor(new Color(color));
	}

	/**
	 * Sets the color of the whiskers, the horizontal lines marking low and high
	 * values. When <code>null</code>, the general series color is used.
	 */
	public void setWhiskerColor(LinearGradient color) {
		setWhiskerColor(new Color(color));
	}

	/**
	 * Sets the color of the whiskers, the horizontal lines marking low and high
	 * values. When <code>null</code>, the general series color is used.
	 */
	public void setWhiskerColor(RadialGradient color) {
		setWhiskerColor(new Color(color));
	}

	/**
	 * Returns the length of the whiskers, the horizontal lines marking low and
	 * high values. It can be a numerical pixel value, or a percentage value of
	 * the box width. Set <code>0</code> to disable whiskers.
	 * <p>
	 * Default: 50%
	 */
	public String getWhiskerLength() {
		return getAttr(Attrs.whiskerLength, "50%").asString();
	}

	/**
	 * Sets the length of the whiskers, the horizontal lines marking low and
	 * high values. It can be a numerical pixel value, or a percentage value of
	 * the box width. Set <code>0</code> to disable whiskers.
	 */
	public void setWhiskerLength(String whiskerLength) {
		setAttr(Attrs.whiskerLength, whiskerLength, "50%");
	}

	/**
	 * Sets the length of the whiskers, the horizontal lines marking low and
	 * high values. It can be a numerical pixel value, or a percentage value of
	 * the box width. Set <code>0</code> to disable whiskers.
	 */
	public void setWhiskerLength(Number whiskerLength) {
		setAttr(Attrs.whiskerLength, whiskerLength);
	}

	/**
	 * Returns the line width of the whiskers, the horizontal lines marking low
	 * and high values. When <code>null</code>, the general
	 * {@link #setLineWidth(Number)} applies.
	 */
	public Number getWhiskerWidth() {
		return getAttr(Attrs.whiskerWidth, 2).asNumber();
	}

	/**
	 * Sets the line width of the whiskers, the horizontal lines marking low and
	 * high values. When <code>null</code>, the general
	 * {@link #setLineWidth(Number)} applies.
	 */
	public void setWhiskerWidth(Number whiskerWidth) {
		setAttr(Attrs.whiskerWidth, whiskerWidth, 2);
	}

	/**
	 * Returns the dash style of the whisker.
	 *
	 * <p>Defaults to Solid.
	 * @since 10.2.1.
	 */
	public String getWhiskerDashStyle() {
		return getAttr(Attrs.whiskerDashStyle, "Solid").asString();
	}

	/**
	 * Sets the dash style of the whisker.
	 *
	 * <p>Defaults to Solid.
	 * @since 10.2.1.
	 */
	public void setWhiskerDashStyle(String whiskerDashStyle) {
		if (!("Solid".equalsIgnoreCase(whiskerDashStyle)
				|| "ShortDash".equalsIgnoreCase(whiskerDashStyle)
				|| "ShortDot".equalsIgnoreCase(whiskerDashStyle)
				|| "ShortDashDot".equalsIgnoreCase(whiskerDashStyle)
				|| "ShortDashDotDot".equalsIgnoreCase(whiskerDashStyle)
				|| "Dot".equalsIgnoreCase(whiskerDashStyle)
				|| "Dash".equalsIgnoreCase(whiskerDashStyle)
				|| "LongDash".equalsIgnoreCase(whiskerDashStyle)
				|| "DashDot".equalsIgnoreCase(whiskerDashStyle)
				|| "LongDashDot".equalsIgnoreCase(whiskerDashStyle) || "LongDashDotDot"
				.equalsIgnoreCase(whiskerDashStyle)))
			throw new IllegalArgumentException("Unsupported style: ["
					+ whiskerDashStyle + "]");
		setAttr(Attrs.whiskerDashStyle, whiskerDashStyle);
	}
}