/* ItemPlotOptions.java

	Purpose:
		
	Description:
		
	History:
		Thu Nov 21 12:30:55 CST 2019, Created by rudyhuang

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart.plotOptions;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import org.zkoss.chart.Chart;
import org.zkoss.chart.Color;
import org.zkoss.chart.LinearGradient;
import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.RadialGradient;
import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.lang.Generics;

/**
 * An option for item plot
 *
 * @author rudyhuang
 * @since 7.2.1.0
 */
public class ItemPlotOptions extends SeriesPlotOptions {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		center, colors, endAngle, fillColor, ignoreHiddenPoint,
		innerSize, itemPadding, layout, minSize, rows, startAngle, size, thickness
	}

	/**
	 * Sets the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 * @since 10.2.1.0
	 */
	public void setColors(List<Color> colors) {
		setAttr(Attrs.colors, colors);
	}

	/**
	 * Sets the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 * @since 10.2.1.0
	 */
	public void setColors(String... colors) {
		setColors(Arrays.stream(colors).map(Color::new).collect(Collectors.toList()));
	}

	/**
	 * Sets the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 * @since 10.2.1.0
	 */
	public void setColors(Color... colors) {
		setColors(Arrays.asList(colors));
	}

	/**
	 * Returns the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 * @since 10.2.1.0
	 */
	public List<Color> getColors() {
		return Generics.cast(getAttr(Attrs.colors, null).asValue());
	}

	/**
	 * Returns the center of the pie chart relative to the plot area.
	 * Can be percentages or pixel values.
	 * The default behaviour is to center the pie so that all slices and data labels are within the plot area.
	 * As a consequence, the pie may actually jump around in a chart with dynamic values,
	 * as the data labels move. In that case, the center should be explicitly set,
	 * for example to ["50%", "50%"].
	 * <p>
	 * Default: [null, null]
	 */
	public List<String> getCenter() {
		if (!containsKey(Attrs.center)) {
			setCenter(Arrays.asList(null, null));
		}
		return Generics.cast(getAttr(Attrs.center));
	}

	/**
	 * Sets the center of the pie chart relative to the plot area.
	 * Can be percentages or pixel values.
	 * The default behaviour is to center the pie so that all slices and data labels are within the plot area.
	 * As a consequence, the pie may actually jump around in a chart with dynamic values,
	 * as the data labels move. In that case, the center should be explicitly set,
	 * for example to ["50%", "50%"].
	 */
	public void setCenter(List<Object> center) {
		setAttr(Attrs.center, center, NOT_NULL_VALUE);
	}

	/**
	 * Sets the center of the pie chart relative to the plot area.
	 * Can be percentages or pixel values.
	 * The default behaviour is to center the pie so that all slices and data labels are within the plot area.
	 * As a consequence, the pie may actually jump around in a chart with dynamic values,
	 * as the data labels move. In that case, the center should be explicitly set,
	 * for example to ["50%", "50%"].
	 */
	public void setCenter(Object... center) {
		setCenter(Arrays.asList(center));
	}

	/**
	 * In circular view, returns the end angle of the item layout, in degrees where 0 is up.
	 * Default: null
	 */
	public Number getEndAngle() {
		return getAttr(Attrs.endAngle, null).asNumber();
	}

	/**
	 * In circular view, sets the end angle of the item layout, in degrees where 0 is up.
	 */
	public void setEndAngle(Number endAngle) {
		setAttr(Attrs.endAngle, endAngle);
	}


	/**
	 * Returns fill color or gradient for the area. When <code>null</code>, the
	 * series' <code>color</code> is used with the series'
	 * <code>fillOpacity</code>.
	 * <p>
	 * Default: null.
	 * @since 10.2.1.0
	 */
	public Color getFillColor() {
		return (Color) getAttr(Attrs.fillColor, null).asValue();
	}

	/**
	 * Sets fill color or gradient for the area. When <code>null</code>, the
	 * series' <code>color</code> is used with the series'
	 * <code>fillOpacity</code>.
	 * @since 10.2.1.0
	 */
	public void setFillColor(Color color) {
		setAttr(Attrs.fillColor, color);
	}

	/**
	 * Sets fill color or gradient for the area. When <code>null</code>, the
	 * series' <code>color</code> is used with the series'
	 * <code>fillOpacity</code>.
	 * @since 10.2.1.0
	 */
	public void setFillColor(String color) {
		setFillColor(new Color(color));
	}

	/**
	 * Sets fill color or gradient for the area. When <code>null</code>, the
	 * series' <code>color</code> is used with the series'
	 * <code>fillOpacity</code>.
	 * @since 10.2.1.0
	 */
	public void setFillColor(LinearGradient color) {
		setFillColor(new Color(color));
	}

	/**
	 * Sets fill color or gradient for the area. When <code>null</code>, the
	 * series' <code>color</code> is used with the series'
	 * <code>fillOpacity</code>.
	 * @since 10.2.1.0
	 */
	public void setFillColor(RadialGradient color) {
		setFillColor(new Color(color));
	}

	/**
	 * Returns whether ignore the hidden point.
	 * <p>
	 * Equivalent to {@link Chart#isIgnoreHiddenSeries()}, this option tells
	 * whether the series shall be redrawn as if the hidden point were
	 * <code>null</code>.
	 * <p>
	 * The default value true.
	 * @since 10.2.1.0
	 */
	public boolean isIgnoreHiddenPoint() {
		return getAttr(Attrs.ignoreHiddenPoint, true).asBoolean();
	}

	/**
	 * Sets whether ignore the hidden point.
	 * <p>
	 * Equivalent to {@link Chart#isIgnoreHiddenSeries()}, this option tells
	 * whether the series shall be redrawn as if the hidden point were
	 * <code>null</code>.
	 * <p>
	 * The default value true.
	 * @since 10.2.1.0
	 */
	public void setIgnoreHiddenPoint(boolean ignoreHiddenPoint) {
		setAttr(Attrs.ignoreHiddenPoint, ignoreHiddenPoint);
	}

	/**
	 * In circular view, returns the size of the inner diameter of the circle.
	 * Can be a percentage or pixel value.
	 * Percentages are relative to the outer perimeter.
	 * Pixel values are given as integers.
	 *
	 * If the {@link #getRows()} option is set, it overrides the innerSize setting.
	 * <p>
	 * Default: "40%"
	 */
	public Object getInnerSize() {
		return getAttr(Attrs.innerSize, "40%").asValue();
	}

	/**
	 * In circular view, sets the size of the inner diameter of the circle.
	 * Can be a percentage.
	 * Percentages are relative to the outer perimeter.
	 *
	 * If the {@link #getRows()} option is set, it overrides the innerSize setting.
	 */
	public void setInnerSize(String innerSize) {
		setAttr(Attrs.innerSize, innerSize);
	}

	/**
	 * In circular view, sets the size of the inner diameter of the circle.
	 * Can be a pixel value.
	 * Pixel values are given as integers.
	 *
	 * If the {@link #getRows()} option is set, it overrides the innerSize setting.
	 */
	public void setInnerSize(int innerSize) {
		setAttr(Attrs.innerSize, innerSize);
	}

	/**
	 * Returns the padding between the items, given in relative size where the size
	 * of the item is 1.
	 * <p>Default: {@code 0.1}
	 * @since 10.2.1.0
	 */
	public Number getItemPadding() {
		return getAttr(Attrs.itemPadding, 0.1).asNumber();
	}

	/**
	 * Sets the padding between the items, given in relative size where the size
	 * of the item is 1.
	 * <p>Default: {@code 0.1}
	 * @since 10.2.1.0
	 */
	public void setItemPadding(Number itemPadding) {
		setAttr(Attrs.itemPadding, itemPadding);
	}

	/**
	 * Returns the layout of the items in rectangular view.
	 * Can be either horizontal or vertical
	 * <p>
	 * Default: "vertical"
	 */
	public String getLayout() {
		return getAttr(Attrs.layout, "vertical").asString();
	}

	/**
	 * Sets the layout of the items in rectangular view.
	 * Can be either horizontal or vertical
	 */
	public void setLayout(String layout) {
		if (!("horizontal".equals(layout) || "vertical".equals(layout)))
			throw new IllegalArgumentException("Unsupported layout: [ " + layout + " ]");
		setAttr(Attrs.layout, layout);
	}

	/**
	 * Returns the number of rows to display in the rectangular or circular view.
	 * If the {@link #getInnerSize()} is set, it will be overridden by the rows setting.
	 * <p>
	 * Default: null
	 */
	public Number getRows() {
		return getAttr(Attrs.rows, null).asNumber();
	}

	/**
	 * Sets the number of rows to display in the rectangular or circular view.
	 * If the {@link #getInnerSize()} is set, it will be overridden by the rows setting.
	 */
	public void setRows(Number rows) {
		setAttr(Attrs.rows, rows);
	}

	/**
	 * Returns the minimum size for a pie in response to auto margins. The pie
	 * will try to shrink to make room for data labels in side the plot area,
	 * but only to this size.
	 * <p>Default: {@code 80}
	 * @since 10.2.1.0
	 */
	public Number getMinSize() {
		return getAttr(Attrs.minSize, 80).asNumber();
	}

	/**
	 * Sets the minimum size for a pie in response to auto margins. The pie will
	 * try to shrink to make room for data labels in side the plot area, but
	 * only to this size.
	 * <p>Default: {@code 80}
	 * @since 10.2.1.0
	 */
	public void setMinSize(Number minSize) {
		setAttr(Attrs.minSize, minSize, 80);
	}

	/**
	 * In circular view, returns the start angle of the item layout, in degrees where 0 is up.
	 * Default: null
	 */
	public Number getStartAngle() {
		return getAttr(Attrs.startAngle, null).asNumber();
	}

	/**
	 * In circular view, sets the start angle of the item layout, in degrees where 0 is up.
	 */
	public void setStartAngle(Number startAngle) {
		setAttr(Attrs.startAngle, startAngle);
	}

	/**
	 * Returns the diameter of the pie relative to the plot area.
	 * Can be a percentage or pixel value. Pixel values are given as integers.
	 * The default behaviour is to scale to the plot area and give room for data
	 * labels within the plot area. slicedOffset is also included in the default size
	 * calculation. As a consequence, the size of the pie may vary when points are
	 * updated and data labels more around. In that case it is best to set a fixed
	 * value, for example {@code "75%"}.
	 *
	 * <p>
	 * Defaults to {@code null}.
	 * @since 10.2.1.0
	 */
	public Object getSize() {
		return getAttr(Attrs.size, null).asValue();
	}

	/**
	 * Sets the diameter of the pie relative to the plot area.
	 * Can be a percentage or pixel value. Pixel values are given as integers.
	 * The default behaviour is to scale to the plot area and give room for data
	 * labels within the plot area. slicedOffset is also included in the default size
	 * calculation. As a consequence, the size of the pie may vary when points are
	 * updated and data labels more around. In that case it is best to set a fixed
	 * value, for example {@code "75%"}.
	 * <p>
	 * Defaults to {@code null}.
	 * @since 10.2.1.0
	 */
	public void setSize(String size) {
		setAttr(Attrs.size, size);
	}

	/**
	 * Sets the diameter of the pie relative to the plot area.
	 * Can be a percentage or pixel value. Pixel values are given as integers.
	 * The default behaviour is to scale to the plot area and give room for data
	 * labels within the plot area. slicedOffset is also included in the default size
	 * calculation. As a consequence, the size of the pie may vary when points are
	 * updated and data labels more around. In that case it is best to set a fixed
	 * value, for example {@code "75%"}.
	 * <p>
	 * Defaults to {@code null}.
	 * @since 10.2.1.0
	 */
	public void setSize(Number size) {
		setAttr(Attrs.size, size);
	}

	/**
	 * Returns thickness describing the ring size for a donut type chart,
	 * overriding {@link #getInnerSize()}.
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public Number getThickness() {
		return getAttr(Attrs.thickness, null).asNumber();
	}

	/**
	 * Sets thickness describing the ring size for a donut type chart,
	 * overriding {@link #getInnerSize()}.
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public void setThickness(Number thickness) {
		setAttr(Attrs.thickness, thickness);
	}
}
