/* NetworkGraphLayoutAlgorithm.java

	Purpose:
		
	Description:
		
	History:
		Fri Nov 15 11:29:03 CST 2019, Created by rudyhuang

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart.plotOptions;

import org.zkoss.chart.Optionable;
import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.json.JavaScriptValue;

/**
 * A option for layout algorithm.
 *
 * @author rudyhuang
 * @since 7.2.1.0
 */
public class NetworkGraphLayoutAlgorithm extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		approximation, attractiveForce, enableSimulation, friction,
		gravitationalConstant, initialPositionRadius, initialPositions,
		integration, linkLength, maxIterations, maxSpeed, repulsiveForce,
		theta, type
	}

	/**
	 * Returns the approximation used to calculate repulsive forces affecting nodes.
	 * By default, when calculating net force, nodes are compared against each other,
	 * which gives O(N^2) complexity.
	 * Using Barnes-Hut approximation, we decrease this to O(N log N),
	 * but the resulting graph will have different layout.
	 * Barnes-Hut approximation divides space into rectangles via quad tree,
	 * where forces exerted on nodes are calculated directly for nearby cells,
	 * and for all others, cells are treated as a separate node with center of mass.
	 * <p>
	 * Default: "none"
	 */
	public String getApproximation() {
		return getAttr(Attrs.approximation, "none").asString();
	}

	/**
	 * Sets the approximation used to calculate repulsive forces affecting nodes.
	 * By default, when calculating net force, nodes are compared against each other,
	 * which gives O(N^2) complexity.
	 * Using Barnes-Hut approximation, we decrease this to O(N log N),
	 * but the resulting graph will have different layout.
	 * Barnes-Hut approximation divides space into rectangles via quad tree,
	 * where forces exerted on nodes are calculated directly for nearby cells,
	 * and for all others, cells are treated as a separate node with center of mass.
	 */
	public void setApproximation(String approximation) {
		setAttr(Attrs.approximation, approximation, "none");
	}

	/**
	 * Returns the attraction force applied on a node which is connected to another node by a link.
	 * Passed are two arguments:
	 * <ul>
	 *   <li>d - which is current distance between two nodes</li>
	 *   <li>k - which is desired distance between two nodes</li>
	 * </ul>
	 * <p>
	 * In <code>verlet</code> integration, defaults to: <code>function (d, k) { return (k - d) / d; }</code>.
	 * Defaults to <code>function (d, k) { return k * k / d; }</code>.
	 */
	public JavaScriptValue getAttractiveForce() {
		return (JavaScriptValue) getAttr(Attrs.attractiveForce);
	}

	/**
	 * Sets the attraction force applied on a node which is connected to another node by a link.
	 * Passed are two arguments:
	 * <ul>
	 *     <li>d - which is current distance between two nodes</li>
	 *     <li>k - which is desired distance between two nodes</li>
	 * </ul>
	 */
	public void setAttractiveForce(JavaScriptValue attractiveForce) {
		setAttr(Attrs.attractiveForce, attractiveForce);
	}

	/**
	 * Experimental.
	 * Returns whether to enable live simulation of the algorithm implementation.
	 * All nodes are animated as the forces applies on them.
	 * <p>
	 * Default: false.
	 */
	public boolean isEnableSimulation() {
		return getAttr(Attrs.enableSimulation, false).asBoolean();
	}

	/**
	 * Experimental.
	 * Sets whether to enable live simulation of the algorithm implementation.
	 * All nodes are animated as the forces applies on them.
	 */
	public void setEnableSimulation(boolean enableSimulation) {
		setAttr(Attrs.enableSimulation, enableSimulation);
	}

	/**
	 * Returns the friction applied on forces to prevent nodes rushing to fast to the desired positions.
	 * <p>
	 * Default: -0.981
	 */
	public Number getFriction() {
		return getAttr(Attrs.friction, -0.981).asNumber();
	}

	/**
	 * Sets the friction applied on forces to prevent nodes rushing to fast to the desired positions.
	 */
	public void setFriction(Number friction) {
		setAttr(Attrs.friction, friction, -0.981);
	}

	/**
	 * Returns the gravitational const used in the barycenter force of the algorithm.
	 * <p>
	 * Default: 0.0625
	 */
	public Number getGravitationalConstant() {
		return getAttr(Attrs.gravitationalConstant, 0.0625).asNumber();
	}

	/**
	 * Sets the gravitational const used in the barycenter force of the algorithm.
	 */
	public void setGravitationalConstant(Number gravitationalConstant) {
		setAttr(Attrs.gravitationalConstant, gravitationalConstant, 0.0625);
	}

	/**
	 * When initialPositions are set to 'circle',
	 * initialPositionRadius is a distance from the center of circle, in which nodes are created.
	 * <p>
	 * Default: 1
	 */
	public Number getInitialPositionRadius() {
		return getAttr(Attrs.initialPositionRadius, 1).asNumber();
	}

	/**
	 * When initialPositions are set to 'circle',
	 * initialPositionRadius is a distance from the center of circle, in which nodes are created.
	 */
	public void setInitialPositionRadius(Number initialPositionRadius) {
		setAttr(Attrs.initialPositionRadius, initialPositionRadius, 1);
	}

	/**
	 * Returns the initial layout algorithm for positioning nodes.
	 * Can be one of built-in options ("circle", "random") or a function where
	 * positions should be set on each node (<code>this.nodes</code>) as <code>node.plotX</code> and <code>node.plotY</code>.
	 * <p>
	 * Default: "circle"
	 */
	public Object getInitialPositions() {
		return getAttr(Attrs.initialPositions, "circle").asValue();
	}

	/**
	 * Sets the initial layout algorithm for positioning nodes.
	 * Can be one of built-in options ("circle", "random").
	 */
	public void setInitialPositions(String initialPositions) {
		if (!("circle".equals(initialPositions) || "random".equals(initialPositions)))
			throw new IllegalArgumentException("Unsupported initialPositions: [" + initialPositions + "]");
		setAttr(Attrs.initialPositions, initialPositions, "circle");
	}

	/**
	 * Sets the initial layout algorithm for positioning nodes.
	 * Can be a function where positions should be set on each node (<code>this.nodes</code>)
	 * as <code>node.plotX</code> and <code>node.plotY</code>.
	 */
	public void setInitialPositions(JavaScriptValue initialPositions) {
		setAttr(Attrs.initialPositions, initialPositions);
	}

	/**
	 * Returns the integration type.
	 * Available options are 'euler' and 'verlet'.
	 * Integration determines how forces are applied on particles.
	 * In Euler integration, force is applied direct as newPosition += velocity;.
	 * In Verlet integration, new position is based on a previous position without velocity: newPosition += previousPosition - newPosition.
	 * Note that different integrations give different results as forces are different.
	 * <p>
	 * Default: "euler"
	 */
	public String getIntegration() {
		return getAttr(Attrs.integration, "euler").asString();
	}

	/**
	 * Sets the integration type.
	 * Available options are 'euler' and 'verlet'.
	 * Integration determines how forces are applied on particles.
	 * In Euler integration, force is applied direct as newPosition += velocity;.
	 * In Verlet integration, new position is based on a previous position without velocity: newPosition += previousPosition - newPosition.
	 * Note that different integrations give different results as forces are different.
	 */
	public void setIntegration(String integration) {
		if (!("euler".equals(integration) || "verlet".equals(integration)))
			throw new IllegalArgumentException("Unsupported integration: [" + integration + "]");
		setAttr(Attrs.integration, integration, "euler");
	}

	/**
	 * Returns the ideal length (px) of the link between two nodes.
	 * When not defined, length is calculated as: <code>Math.pow(availableWidth * availableHeight / nodesLength, 0.4);</code>
	 * Note: Because of the algorithm specification, length of each link might be not exactly as specified.
	 * <p>
	 * Default: null
	 */
	public Number getLinkLength() {
		return getAttr(Attrs.linkLength, null).asNumber();
	}

	/**
	 * Sets the ideal length (px) of the link between two nodes.
	 * When not defined, length is calculated as: <code>Math.pow(availableWidth * availableHeight / nodesLength, 0.4);</code>
	 * Note: Because of the algorithm specification, length of each link might be not exactly as specified.
	 */
	public void setLinkLength(Number linkLength) {
		setAttr(Attrs.linkLength, linkLength);
	}

	/**
	 * Returns the max number of iterations before algorithm will stop.
	 * In general, algorithm should find positions sooner, but when rendering
	 * huge number of nodes, it is recommended to increase this value as finding
	 * perfect graph positions can require more time.
	 * <p>
	 * Default: 1000
	 */
	public Number getMaxIterations() {
		return getAttr(Attrs.maxIterations, 1000).asNumber();
	}

	/**
	 * Sets the max number of iterations before algorithm will stop.
	 * In general, algorithm should find positions sooner, but when rendering
	 * huge number of nodes, it is recommended to increase this value as finding
	 * perfect graph positions can require more time.
	 */
	public void setMaxIterations(Number maxIterations) {
		setAttr(Attrs.maxIterations, maxIterations, 1000);
	}

	/**
	 * Verlet integration only.
	 * Returns the max speed that node can get in one iteration.
	 * In terms of simulation, it's a maximum translation (in pixels) that node
	 * can move (in both, x and y, dimensions). While friction is applied on all
	 * nodes, max speed is applied only for nodes that move very fast, for example
	 * small or disconnected ones.
	 * <p>
	 * Default: 10
	 */
	public Number getMaxSpeed() {
		return getAttr(Attrs.maxSpeed, 10).asNumber();
	}

	/**
	 * Verlet integration only.
	 * Sets the max speed that node can get in one iteration.
	 * In terms of simulation, it's a maximum translation (in pixels) that node
	 * can move (in both, x and y, dimensions). While friction is applied on all
	 * nodes, max speed is applied only for nodes that move very fast, for example
	 * small or disconnected ones.
	 */
	public void setMaxSpeed(Number maxSpeed) {
		setAttr(Attrs.maxSpeed, maxSpeed, 10);
	}

	/**
	 * Returns the repulsive force applied on a node.
	 * Passed are two arguments:
	 * <ul>
	 *     <li>d - which is current distance between two nodes</li>
	 *     <li>k - which is desired distance between two nodes</li>
	 * </ul>
	 * <p>
	 * In <code>verlet</code> integration, defaults to: {@code function (d, k) { return (k - d) / d * (k > d ? 1 : 0) }}.
	 * Defaults to <code>function (d, k) { return k * k / d; }</code>.
	 */
	public JavaScriptValue getRepulsiveForce() {
		return (JavaScriptValue) getAttr(Attrs.repulsiveForce);
	}

	/**
	 * Sets the repulsive force applied on a node.
	 * Passed are two arguments:
	 * <ul>
	 *     <li>d - which is current distance between two nodes</li>
	 *     <li>k - which is desired distance between two nodes</li>
	 * </ul>
	 */
	public void setRepulsiveForce(JavaScriptValue repulsiveForce) {
		setAttr(Attrs.repulsiveForce, repulsiveForce);
	}

	/**
	 * Barnes-Hut approximation only.
	 * Determines when distance between cell and node is small enough to calculate forces.
	 * Value of theta is compared directly with quotient <code>s / d</code>,
	 * where <code>s</code> is the size of the cell, and <code>d</code> is distance
	 * between center of cell's mass and currently compared node.
	 * <p>
	 * Default: 0.5
	 */
	public Number getTheta() {
		return getAttr(Attrs.theta, 0.5).asNumber();
	}

	/**
	 * Barnes-Hut approximation only.
	 * Determines when distance between cell and node is small enough to calculate forces.
	 * Value of theta is compared directly with quotient <code>s / d</code>,
	 * where <code>s</code> is the size of the cell, and <code>d</code> is distance
	 * between center of cell's mass and currently compared node.
	 */
	public void setTheta(Number theta) {
		setAttr(Attrs.theta, theta, 0.5);
	}

	/**
	 * Returns the type of the algorithm used when positioning nodes.
	 * <p>
	 * Default: "reingold-fruchterman"
	 */
	public String getType() {
		return getAttr(Attrs.type, "reingold-fruchterman").asString();
	}

	/**
	 * Sets the type of the algorithm used when positioning nodes.
	 */
	public void setType(String type) {
		setAttr(Attrs.type, type, "reingold-fruchterman");
	}
}