/* TreegraphPlotOptions.java

	Purpose:
		
	Description:
		
	History:
		10:47 AM 2023/4/7, Created by jumperchen

Copyright (C) 2023 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart.plotOptions;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import org.zkoss.chart.Color;
import org.zkoss.chart.Level;
import org.zkoss.chart.Tooltip;
import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.TraverseUpButton;
import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.lang.Generics;

/**
 * Options for treegraph.
 * @author jumperchen
 * @since 10.3.3.0
 */
public class TreegraphPlotOptions extends SeriesPlotOptions {

	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		animationLimit,
		allowTraversingTree,
		collapseButton,
		colors,
		fillSpace,
		findNearestPointBy,
		levels,
		link,
		opacity,
		reversed,
		showInLegend,
		stickyTracking,
		traverseUpButton
	}

	/**
	 * When enabled the user can click on a point which is a parent and zoom in
	 * on its children.
	 * <p>
	 * Defaults to false.
	 */
	public boolean isAllowTraversingTree() {
		return getAttr(Attrs.allowTraversingTree, false).asBoolean();
	}
	/**
	 * @param allowTraversingTree
	 * @see #isAllowTraversingTree()
	 */
	public void setAllowTraversingTree(boolean allowTraversingTree) {
		setAttr(Attrs.allowTraversingTree, allowTraversingTree, false);
	}

	/**
	 * For some series, there is a limit that shuts down initial animation by default
	 * when the total number of points in the chart is too high.
	 * For example, for a column chart and its derivatives, animation does not run
	 * if there is more than 250 points totally.
	 * To disable this cap, set animationLimit to Infinity.
	 * <p>Default: 250
	 * @since 11.2.0.0
	 */
	public Number getAnimationLimit() {
		return getAttr(Attrs.animationLimit, 250).asNumber();
	}

	/**
	 * For some series, there is a limit that shuts down initial animation by default
	 * when the total number of points in the chart is too high.
	 * For example, for a column chart and its derivatives, animation does not run
	 * if there is more than 250 points totally.
	 * To disable this cap, set animationLimit to Infinity.
	 * <p>Default: 250
	 * @since 11.2.0.0
	 */
	public void setAnimationLimit(Number animationLimit) {
		setAttr(Attrs.animationLimit, animationLimit, 250);
	}

	/**
	 * Returns the options applied to collapse Button.
	 */
	public CollapseButton getCollapseButton() {
		CollapseButton options = (CollapseButton) getAttr(Attrs.collapseButton);
		 if (options == null) {
			 options = new CollapseButton();
			 setCollapseButton(options);
		 }
		 return options;
	}

	/**
	 * Sets the options applied to collapse Button.
	 */
	public void setCollapseButton(CollapseButton collapseButton) {
		setAttr(Attrs.collapseButton, collapseButton);
	}


	/**
	 * Sets the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 */
	public void setColors(List<Color> colors) {
		setAttr(Attrs.colors, colors);
	}

	/**
	 * Sets the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 */
	public void setColors(String... colors) {
		setColors(Arrays.stream(colors).map(Color::new).collect(Collectors.toList()));
	}

	/**
	 * Sets the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 */
	public void setColors(Color... colors) {
		setColors(Arrays.asList(colors));
	}

	/**
	 * Returns the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 */
	public List<Color> getColors() {
		return Generics.cast(getAttr(Attrs.colors, null).asValue());
	}

	/**
	 * Returns whether the treegraph series should fill the entire plot area in the X axis direction,
	 * even when there are collapsed points.
	 * <p>
	 * Defaults: false
	 * @since 11.2.0.0
	 */
	public boolean isFillSpace() {
		return getAttr(Attrs.fillSpace, false).asBoolean();
	}
	/**
	 * Sets whether the treegraph series should fill the entire plot area in the X axis direction,
	 * even when there are collapsed points.
	 * <p>
	 * @since 11.2.0.0
	 */
	public void setFillSpace(boolean fillSpace) {
		setAttr(Attrs.fillSpace, fillSpace, false);
	}

	/**
	 * Returns whether the series should look for the nearest point in both
	 * dimensions or just the x-dimension when hovering the series.
	 * Defaults to 'xy' for scatter series and 'x' for most other series.
	 * If the data has duplicate x-values, it is recommended to set this
	 * to 'xy' to allow hovering over all points.
	 *
	 * Applies only to series types using nearest neighbor search (not direct hover) for tooltip.
	 * <p>
	 * Default: "xy"
	 * @since 11.2.0.0
	 */
	public String getFindNearestPointBy() {
		return getAttr(Attrs.findNearestPointBy, "xy").asString();
	}

	/**
	 * Sets whether the series should look for the nearest point in both
	 * dimensions or just the x-dimension when hovering the series.
	 * Defaults to 'xy' for scatter series and 'x' for most other series.
	 * If the data has duplicate x-values, it is recommended to set this
	 * to 'xy' to allow hovering over all points.
	 *
	 * Applies only to series types using nearest neighbor search (not direct hover) for tooltip.
	 * @since 11.2.0.0
	 */
	public void setFindNearestPointBy(String findNearestPointBy) {
		setAttr(Attrs.findNearestPointBy, findNearestPointBy);
	}


	/**
	 * Set options on specific levels. Takes precedence over series options,
	 * but not point options.
	 * <p>
	 * Default: null
	 */
	public List<Level> getLevels() {
		return Generics.cast(getAttr(Attrs.levels, null).asValue());
	}

	/**
	 * @param levels
	 * @see #getLevels()
	 */
	public void setLevels(List<Level> levels) {
		setAttr(Attrs.levels, levels);
	}

	/**
	 * Set options on specific levels. Takes precedence over series options,
	 * but not point options.
	 */
	public void setLevels(Level... levels) {
		setLevels(Arrays.asList(levels));
	}


	/**
	 * Returns the link style options, if any. Otherwise, create a new one.
	 */
	public TreeGraphLink getLink() {
		TreeGraphLink link = (TreeGraphLink) getAttr(Attrs.link);
		if (link == null) {
			link = new TreeGraphLink();
			setLink(link);
		}
		return link;
	}

	/**
	 * Sets the link style options.
	 */
	public void setLink(TreeGraphLink link) {
		setAttr(Attrs.link, link);
	}

	/**
	 * Returns opacity of a series parts: line, fill (e.g. area) and dataLabels.
	 * <p>
	 * Default: 0.15
	 * @since 11.2.0.0
	 */
	public Number getOpacity() {
		return getAttr(Attrs.opacity, 0.15).asNumber();
	}

	/**
	 * Sets opacity of a series parts: line, fill (e.g. area) and dataLabels.
	 * @since 11.2.0.0
	 */
	public void setOpacity(Number opacity) {
		setAttr(Attrs.opacity, opacity);
	}

	/**
	 * Returns whether the series should be placed on the other side of the plotArea.
	 *
	 * <p>
	 * Default: {@code false}.
	 * @since 11.2.0.0
	 */
	public boolean isReversed() {
		return getAttr(Attrs.reversed, false).asBoolean();
	}

	/**
	 * Sets whether the series should be placed on the other side of the plotArea.
	 * @since 11.2.0.0
	 */
	public void setReversed(boolean reversed) {
		setAttr(Attrs.reversed, reversed);
	}

	/**
	 * Returns whether to display this particular series or series type in the
	 * legend. The default value is <code>true</code> for standalone series,
	 * <code>false</code> for linked series.
	 * Default: {@code false}.
	 * @since 11.2.0.0
	 */
	public boolean isShowInLegend() {
		return getAttr(Attrs.showInLegend, false).asBoolean();
	}

	/**
	 * Sets whether to display this particular series or series type in the
	 * legend. The default value is <code>true</code> for standalone series,
	 * <code>false</code> for linked series.
	 * @since 11.2.0.0
	 */
	public void setShowInLegend(boolean showInLegend) {
		setAttr(Attrs.showInLegend, showInLegend);
	}

	/**
	 * Returns sticky tracking of mouse events. When true, the
	 * <code>mouseOut</code> event on a series isn't triggered until the mouse
	 * moves over another series, or out of the plot area. When false, the
	 * <code>mouseOut</code> event on a series is triggered when the mouse
	 * leaves the area around the series' graph or markers. This also implies
	 * the tooltip. When <code>stickyTracking</code> is false and
	 * {@link Tooltip#isShared()} is false, the tooltip will be hidden when
	 * moving the mouse between series. Defaults to true for line and area type
	 * series, but to false for columns, pies etc.
	 * Default: {@code false}.
	 * @since 11.2.0.0
	 */
	public boolean isStickyTracking() {
		return getAttr(Attrs.stickyTracking, false).asBoolean();
	}

	/**
	 * Sets sticky tracking of mouse events. When true, the
	 * <code>mouseOut</code> event on a series isn't triggered until the mouse
	 * moves over another series, or out of the plot area. When false, the
	 * <code>mouseOut</code> event on a series is triggered when the mouse
	 * leaves the area around the series' graph or markers. This also implies
	 * the tooltip. When <code>stickyTracking</code> is false and
	 * {@link Tooltip#isShared()} is false, the tooltip will be hidden when
	 * moving the mouse between series. Defaults to true for line and area type
	 * series, but to false for columns, pies etc.
	 * @since 11.2.0.0
	 */
	public void setStickyTracking(boolean stickyTracking) {
		setAttr(Attrs.stickyTracking, stickyTracking);
	}



	/**
	 * Returns options for the button appearing when traversing down in a treegraph, if any.
	 * Otherwise, create a new one.
	 */
	public TraverseUpButton getTraverseUpButton() {
		TraverseUpButton traverseUpButton = (TraverseUpButton) getAttr(Attrs.traverseUpButton);
		if (traverseUpButton == null) {
			traverseUpButton = new TraverseUpButton();
			setTraverseUpButton(traverseUpButton);
		}
		return traverseUpButton;
	}

	/**
	 * Sets options for the button appearing when traversing down in a treemap.
	 */
	public void setTraverseUpButton(TraverseUpButton traverseUpButton) {
		setAttr(Attrs.traverseUpButton, traverseUpButton);
	}
}