/* WordCloudPlotOptions.java

	Purpose:
		
	Description:
		
	History:
		Fri Dec 06 15:29:37 CST 2019, Created by rudyhuang

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart.plotOptions;

import java.util.LinkedHashMap;
import java.util.Map;

import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.chart.util.MapsHelper;
import org.zkoss.lang.Generics;

/**
 * An option for wordcloud plot
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 *
 * @author rudyhuang
 * @since 7.2.1.0
 */
public class WordCloudPlotOptions extends ColumnPlotOptions {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		borderWidth, edgeWidth,
		maxFontSize, minFontSize, placementStrategy, rotation, spiral, style
	}
	/**
	 * Returns the width of the border surrounding each column or bar.
	 * <p>
	 * Default: {@code 0} when the columns are so dense that a border would cover the next column.
	 * @since 10.2.1.0
	 */
	public Number getBorderWidth() {
		return getAttr(Attrs.borderWidth, 0).asNumber();
	}

	/**
	 * Sets the width of the border surrounding each column or bar.
	 * @since 10.2.1.0
	 */
	public void setBorderWidth(Number borderWidth) {
		setAttr(Attrs.borderWidth, borderWidth);
	}

	/**
	 * 3D columns only.
	 * Returns the width of the colored edges.
	 * <p>
	 * Default: 1
	 * @since 10.2.1.0
	 */
	public Number getEdgeWidth() {
		return getAttr(Attrs.edgeWidth, 1).asNumber();
	}

	/**
	 * 3D columns only.
	 * Sets the width of the colored edges.
	 * @since 10.2.1.0
	 */
	public void setEdgeWidth(Number edgeWidth) {
		setAttr(Attrs.edgeWidth, edgeWidth);
	}

	/**
	 * Returns the word with the largest weight will have a font size equal to this value.
	 * The font size of a word is the ratio between its weight and the largest occurring weight,
	 * multiplied with the value of maxFontSize.
	 * <p>
	 * Default: 25
	 */
	public Number getMaxFontSize() {
		return getAttr(Attrs.maxFontSize, 25).asNumber();
	}

	/**
	 * Sets the word with the largest weight will have a font size equal to this value.
	 * The font size of a word is the ratio between its weight and the largest occurring weight,
	 * multiplied with the value of maxFontSize.
	 */
	public void setMaxFontSize(Number maxFontSize) {
		setAttr(Attrs.maxFontSize, maxFontSize);
	}

	/**
	 * Returns a threshold determining the minimum font size that can be applied to a word.
	 * <p>
	 * Default: 1
	 */
	public Number getMinFontSize() {
		return getAttr(Attrs.minFontSize, 1).asNumber();
	}

	/**
	 * Sets a threshold determining the minimum font size that can be applied to a word.
	 */
	public void setMinFontSize(Number minFontSize) {
		setAttr(Attrs.minFontSize, minFontSize);
	}

	/**
	 * Returns which algorithm is used for placement, and rotation of a word.
	 * The choice of algorithm is therefore a crucial part of the resulting layout of the wordcloud.
	 * It is possible for users to add their own custom placement strategies for use in word cloud.
	 * Read more about it in <a href="https://www.highcharts.com/docs/chart-and-series-types/word-cloud-series#custom-placement-strategies">documentation</a>.
	 * <p>
	 * Default: "center"
	 */
	public String getPlacementStrategy() {
		return getAttr(Attrs.placementStrategy, "center").asString();
	}

	/**
	 * Sets which algorithm is used for placement, and rotation of a word.
	 * The choice of algorithm is therefore a crucial part of the resulting layout of the wordcloud.
	 * It is possible for users to add their own custom placement strategies for use in word cloud.
	 * Read more about it in <a href="https://www.highcharts.com/docs/chart-and-series-types/word-cloud-series#custom-placement-strategies">documentation</a>.
	 */
	public void setPlacementStrategy(String placementStrategy) {
		setAttr(Attrs.placementStrategy, placementStrategy);
	}

	/**
	 * Returns the rotation options for the words in the wordcloud, if any.
	 * Otherwise, create a new one.
	 */
	public WordCloudRotation getRotation() {
		WordCloudRotation rotation = (WordCloudRotation) getAttr(Attrs.rotation);
		if (rotation == null) {
			rotation = new WordCloudRotation();
			setRotation(rotation);
		}
		return rotation;
	}

	/**
	 * Sets the rotation options for the words in the wordcloud.
	 */
	public void setRotation(WordCloudRotation rotation) {
		setAttr(Attrs.rotation, rotation);
	}

	/**
	 * Returns the spiral used for placing a word after the initial position
	 * experienced a collision with either another word or the borders.
	 * It is possible for users to add their own custom spiralling algorithms
	 * for use in word cloud. Read more about it in <a href="https://www.highcharts.com/docs/chart-and-series-types/word-cloud-series#custom-spiralling-algorithm">documentation</a>.
	 * <p>
	 * Default: "rectangular"
	 */
	public String getSpiral() {
		return getAttr(Attrs.spiral, "rectangular").asString();
	}

	/**
	 * Sets the spiral used for placing a word after the initial position
	 * experienced a collision with either another word or the borders.
	 * It is possible for users to add their own custom spiralling algorithms
	 * for use in word cloud. Read more about it in <a href="https://www.highcharts.com/docs/chart-and-series-types/word-cloud-series#custom-spiralling-algorithm">documentation</a>.
	 */
	public void setSpiral(String spiral) {
		setAttr(Attrs.spiral, spiral);
	}

	/**
	 * Returns CSS styles for the words.
	 * <p>
	 * Default: "fontFamily: 'sans-serif'; fontWeight: 900"
	 * @param <K> type for the key
	 * <p>
	 * if style was set using {@link #setStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 * @param <V> type for the value
	 * <p>
	 * if style was set using {@link #setStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 */
	public <K, V> Map<K, V> getStyle() {
		if (!containsKey(Attrs.style)) {
			setStyle("fontFamily: 'sans-serif'; fontWeight: 900");
		}
		return Generics.cast(getAttr(Attrs.style));
	}

	/**
	 * Sets CSS styles for the words.
	 * <p>
	 * This method allows <b>simple</b> settings like
	 * <pre>{@code
	 * label.setStyle("fontWeight: 'bold'; color: 'white'; textShadow: '0px 1px 2px black'");
	 * }</pre>
	 * <p>
	 * In the format of CSS key:value, and ";" separating multiple key:value pairs,
	 * if you have more a complex CSS styling, we recommend using {@link #setStyle(Map)}
	 * @param style one or many CSS key:value pairs in a single line string format
	 */
	public void setStyle(String style) {
		setStyle(MapsHelper.parse(new LinkedHashMap<String, String>(), style, ':', ';', '\''));
	}

	/**
	 * Sets CSS styles for the words.
	 * <p>
	 * This method allows <b>complex</b> styling like:
	 * <pre>
	 * {@code
	 * Map<String, Object> styles = new HashMap<String, Object>();
	 * styles.put("fill", "#E8E8EA");
	 * styles.put("stroke", "#6E6E78");
	 * styles.put("stroke-width", "1");
	 * styles.put("r", "0");
	 * Map<String, Object> states = new HashMap<String, Object>();
	 * Map<String, String> hover = new HashMap<String, String>();
	 * hover.put("fill", "#F4F4F6");
	 * hover.put("stroke", "#6E6E78");
	 * hover.put("stroke-width", "1");
	 * hover.put("r", "0");
	 * states.put("hover", hover);
	 * styles.put("states", states);
	 * Map<String, String> style = new HashMap<String, String>();
	 * style.put("color", "#4D5663");
	 * style.put("fontSize", "12px");
	 * style.put("fontFamily", "'Arial'");
	 * style.put("fontWeight", "normal");
	 * style.put("padding", "10px");
	 * styles.put("style", style);
	 * }</pre>
	 * which will generate the final results like:
	 * <pre>{@code
	 * {
	 *     "fill": "#E8E8EA",
	 *     "stroke": "#6E6E78",
	 *     "stroke-width": "1",
	 *     "r": "0",
	 *     "states":
	 *     {
	 *         "hover":
	 *         {
	 *             "fill": "#F4F4F6",
	 *             "stroke": "#6E6E78",
	 *             "stroke-width": "1",
	 *             "r": "0"
	 *         }
	 *     },
	 *     "style":
	 *     {
	 *         "color": "#4D5663",
	 *         "fontSize": "12px",
	 *         "fontFamily": "'Arial'",
	 *         "fontWeight": "normal",
	 *         "padding": "10px"
	 *     }
	 * }
	 * }</pre>
	 * @param <K> Type for the key, usually String
	 * @param <V> Type for the value, usually String. For nested maps, use Object or any other type.
	 * @param style the style object
	 */
	public <K, V> void setStyle(Map<K, V> style) {
		setAttr(Attrs.style, style, NOT_NULL_VALUE);
	}
}
